# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import google.protobuf.timestamp_pb2 as timestamp_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.gkehub.v1",
    manifest={
        "Membership",
        "MembershipEndpoint",
        "KubernetesResource",
        "ResourceOptions",
        "ResourceManifest",
        "GkeCluster",
        "KubernetesMetadata",
        "MonitoringConfig",
        "MembershipState",
        "Authority",
    },
)


class Membership(proto.Message):
    r"""Membership contains information about a member cluster.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        endpoint (google.cloud.gkehub_v1.types.MembershipEndpoint):
            Optional. Endpoint information to reach this
            member.

            This field is a member of `oneof`_ ``type``.
        name (str):
            Output only. The full, unique name of this Membership
            resource in the format
            ``projects/*/locations/*/memberships/{membership_id}``, set
            during creation.

            ``membership_id`` must be a valid RFC 1123 compliant DNS
            label:

            1. At most 63 characters in length
            2. It must consist of lower case alphanumeric characters or
               ``-``
            3. It must start and end with an alphanumeric character

            Which can be expressed as the regex:
            ``[a-z0-9]([-a-z0-9]*[a-z0-9])?``, with a maximum length of
            63 characters.
        labels (MutableMapping[str, str]):
            Optional. Labels for this membership.
        description (str):
            Output only. Description of this membership, limited to 63
            characters. Must match the regex:
            ``[a-zA-Z0-9][a-zA-Z0-9_\-\.\ ]*``

            This field is present for legacy purposes.
        state (google.cloud.gkehub_v1.types.MembershipState):
            Output only. State of the Membership
            resource.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. When the Membership was created.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. When the Membership was last
            updated.
        delete_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. When the Membership was deleted.
        external_id (str):
            Optional. An externally-generated and managed ID for this
            Membership. This ID may be modified after creation, but this
            is not recommended.

            The ID must match the regex:
            ``[a-zA-Z0-9][a-zA-Z0-9_\-\.]*``

            If this Membership represents a Kubernetes cluster, this
            value should be set to the UID of the ``kube-system``
            namespace object.
        last_connection_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. For clusters using Connect, the
            timestamp of the most recent connection
            established with Google Cloud. This time is
            updated every several minutes, not continuously.
            For clusters that do not use GKE Connect, or
            that have never connected successfully, this
            field will be unset.
        unique_id (str):
            Output only. Google-generated UUID for this resource. This
            is unique across all Membership resources. If a Membership
            resource is deleted and another resource with the same name
            is created, it gets a different unique_id.
        authority (google.cloud.gkehub_v1.types.Authority):
            Optional. How to identify workloads from this
            Membership. See the documentation on Workload
            Identity for more details:

            https://cloud.google.com/kubernetes-engine/docs/how-to/workload-identity
        monitoring_config (google.cloud.gkehub_v1.types.MonitoringConfig):
            Optional. The monitoring config information
            for this membership.
    """

    endpoint: "MembershipEndpoint" = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="type",
        message="MembershipEndpoint",
    )
    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=2,
    )
    description: str = proto.Field(
        proto.STRING,
        number=3,
    )
    state: "MembershipState" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="MembershipState",
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=6,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=7,
        message=timestamp_pb2.Timestamp,
    )
    delete_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=8,
        message=timestamp_pb2.Timestamp,
    )
    external_id: str = proto.Field(
        proto.STRING,
        number=9,
    )
    last_connection_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=10,
        message=timestamp_pb2.Timestamp,
    )
    unique_id: str = proto.Field(
        proto.STRING,
        number=11,
    )
    authority: "Authority" = proto.Field(
        proto.MESSAGE,
        number=12,
        message="Authority",
    )
    monitoring_config: "MonitoringConfig" = proto.Field(
        proto.MESSAGE,
        number=14,
        message="MonitoringConfig",
    )


class MembershipEndpoint(proto.Message):
    r"""MembershipEndpoint contains information needed to contact a
    Kubernetes API, endpoint and any additional Kubernetes metadata.

    Attributes:
        gke_cluster (google.cloud.gkehub_v1.types.GkeCluster):
            Optional. GKE-specific information. Only
            present if this Membership is a GKE cluster.
        kubernetes_metadata (google.cloud.gkehub_v1.types.KubernetesMetadata):
            Output only. Useful Kubernetes-specific
            metadata.
        kubernetes_resource (google.cloud.gkehub_v1.types.KubernetesResource):
            Optional. The in-cluster Kubernetes Resources that should be
            applied for a correctly registered cluster, in the steady
            state. These resources:

            - Ensure that the cluster is exclusively registered to one
              and only one Hub Membership.
            - Propagate Workload Pool Information available in the
              Membership Authority field.
            - Ensure proper initial configuration of default Hub
              Features.
        google_managed (bool):
            Output only. Whether the lifecycle of this
            membership is managed by a google cluster
            platform service.
    """

    gke_cluster: "GkeCluster" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="GkeCluster",
    )
    kubernetes_metadata: "KubernetesMetadata" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="KubernetesMetadata",
    )
    kubernetes_resource: "KubernetesResource" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="KubernetesResource",
    )
    google_managed: bool = proto.Field(
        proto.BOOL,
        number=8,
    )


class KubernetesResource(proto.Message):
    r"""KubernetesResource contains the YAML manifests and
    configuration for Membership Kubernetes resources in the
    cluster. After CreateMembership or UpdateMembership, these
    resources should be re-applied in the cluster.

    Attributes:
        membership_cr_manifest (str):
            Input only. The YAML representation of the
            Membership CR. This field is ignored for GKE
            clusters where Hub can read the CR directly.

            Callers should provide the CR that is currently
            present in the cluster during CreateMembership
            or UpdateMembership, or leave this field empty
            if none exists. The CR manifest is used to
            validate the cluster has not been registered
            with another Membership.
        membership_resources (MutableSequence[google.cloud.gkehub_v1.types.ResourceManifest]):
            Output only. Additional Kubernetes resources
            that need to be applied to the cluster after
            Membership creation, and after every update.

            This field is only populated in the Membership
            returned from a successful long-running
            operation from CreateMembership or
            UpdateMembership. It is not populated during
            normal GetMembership or ListMemberships
            requests. To get the resource manifest after the
            initial registration, the caller should make a
            UpdateMembership call with an empty field mask.
        connect_resources (MutableSequence[google.cloud.gkehub_v1.types.ResourceManifest]):
            Output only. The Kubernetes resources for
            installing the GKE Connect agent
            This field is only populated in the Membership
            returned from a successful long-running
            operation from CreateMembership or
            UpdateMembership. It is not populated during
            normal GetMembership or ListMemberships
            requests. To get the resource manifest after the
            initial registration, the caller should make a
            UpdateMembership call with an empty field mask.
        resource_options (google.cloud.gkehub_v1.types.ResourceOptions):
            Optional. Options for Kubernetes resource
            generation.
    """

    membership_cr_manifest: str = proto.Field(
        proto.STRING,
        number=1,
    )
    membership_resources: MutableSequence["ResourceManifest"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="ResourceManifest",
    )
    connect_resources: MutableSequence["ResourceManifest"] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="ResourceManifest",
    )
    resource_options: "ResourceOptions" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="ResourceOptions",
    )


class ResourceOptions(proto.Message):
    r"""ResourceOptions represent options for Kubernetes resource
    generation.

    Attributes:
        connect_version (str):
            Optional. The Connect agent version to use for
            connect_resources. Defaults to the latest GKE Connect
            version. The version must be a currently supported version,
            obsolete versions will be rejected.
        v1beta1_crd (bool):
            Optional. Use ``apiextensions/v1beta1`` instead of
            ``apiextensions/v1`` for CustomResourceDefinition resources.
            This option should be set for clusters with Kubernetes
            apiserver versions <1.16.
        k8s_version (str):
            Optional. Major and minor version of the Kubernetes cluster.
            This is only used to determine which version to use for the
            CustomResourceDefinition resources,
            ``apiextensions/v1beta1`` or\ ``apiextensions/v1``.
        k8s_git_version (str):
            Optional. Git version of the Kubernetes
            cluster. This is only used to gate the Connect
            Agent migration to svc.id.goog on GDC-SO
            1.33.100 patch and above.
    """

    connect_version: str = proto.Field(
        proto.STRING,
        number=1,
    )
    v1beta1_crd: bool = proto.Field(
        proto.BOOL,
        number=2,
    )
    k8s_version: str = proto.Field(
        proto.STRING,
        number=3,
    )
    k8s_git_version: str = proto.Field(
        proto.STRING,
        number=4,
    )


class ResourceManifest(proto.Message):
    r"""ResourceManifest represents a single Kubernetes resource to
    be applied to the cluster.

    Attributes:
        manifest (str):
            YAML manifest of the resource.
        cluster_scoped (bool):
            Whether the resource provided in the manifest is
            ``cluster_scoped``. If unset, the manifest is assumed to be
            namespace scoped.

            This field is used for REST mapping when applying the
            resource in a cluster.
    """

    manifest: str = proto.Field(
        proto.STRING,
        number=1,
    )
    cluster_scoped: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class GkeCluster(proto.Message):
    r"""GkeCluster contains information specific to GKE clusters.

    Attributes:
        resource_link (str):
            Immutable. Self-link of the Google Cloud
            resource for the GKE cluster. For example:

            //container.googleapis.com/projects/my-project/locations/us-west1-a/clusters/my-cluster

            Zonal clusters are also supported.
        cluster_missing (bool):
            Output only. If cluster_missing is set then it denotes that
            the GKE cluster no longer exists in the GKE Control Plane.
    """

    resource_link: str = proto.Field(
        proto.STRING,
        number=1,
    )
    cluster_missing: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class KubernetesMetadata(proto.Message):
    r"""KubernetesMetadata provides informational metadata for
    Memberships representing Kubernetes clusters.

    Attributes:
        kubernetes_api_server_version (str):
            Output only. Kubernetes API server version string as
            reported by ``/version``.
        node_provider_id (str):
            Output only. Node providerID as reported by the first node
            in the list of nodes on the Kubernetes endpoint. On
            Kubernetes platforms that support zero-node clusters (like
            GKE-on-GCP), the node_count will be zero and the
            node_provider_id will be empty.
        node_count (int):
            Output only. Node count as reported by
            Kubernetes nodes resources.
        vcpu_count (int):
            Output only. vCPU count as reported by
            Kubernetes nodes resources.
        memory_mb (int):
            Output only. The total memory capacity as
            reported by the sum of all Kubernetes nodes
            resources, defined in MB.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time at which these details were last
            updated. This update_time is different from the
            Membership-level update_time since EndpointDetails are
            updated internally for API consumers.
    """

    kubernetes_api_server_version: str = proto.Field(
        proto.STRING,
        number=1,
    )
    node_provider_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    node_count: int = proto.Field(
        proto.INT32,
        number=3,
    )
    vcpu_count: int = proto.Field(
        proto.INT32,
        number=4,
    )
    memory_mb: int = proto.Field(
        proto.INT32,
        number=5,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=100,
        message=timestamp_pb2.Timestamp,
    )


class MonitoringConfig(proto.Message):
    r"""This field informs Fleet-based applications/services/UIs with
    the necessary information for where each underlying Cluster
    reports its metrics.

    Attributes:
        project_id (str):
            Immutable. Project used to report Metrics
        location (str):
            Immutable. Location used to report Metrics
        cluster (str):
            Immutable. Cluster name used to report metrics. For Anthos
            on VMWare/Baremetal, it would be in format
            ``memberClusters/cluster_name``; And for Anthos on
            MultiCloud, it would be in format
            ``{azureClusters, awsClusters}/cluster_name``.
        kubernetes_metrics_prefix (str):
            Kubernetes system metrics, if available, are
            written to this prefix. This defaults to
            kubernetes.io for GKE, and kubernetes.io/anthos
            for Anthos eventually. Noted: Anthos MultiCloud
            will have kubernetes.io prefix today but will
            migration to be under kubernetes.io/anthos
        cluster_hash (str):
            Immutable. Cluster hash, this is a unique
            string generated by google code, which does not
            contain any PII, which we can use to reference
            the cluster. This is expected to be created by
            the monitoring stack and persisted into the
            Cluster object as well as to GKE-Hub.
    """

    project_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    location: str = proto.Field(
        proto.STRING,
        number=2,
    )
    cluster: str = proto.Field(
        proto.STRING,
        number=3,
    )
    kubernetes_metrics_prefix: str = proto.Field(
        proto.STRING,
        number=4,
    )
    cluster_hash: str = proto.Field(
        proto.STRING,
        number=5,
    )


class MembershipState(proto.Message):
    r"""MembershipState describes the state of a Membership resource.

    Attributes:
        code (google.cloud.gkehub_v1.types.MembershipState.Code):
            Output only. The current state of the
            Membership resource.
    """

    class Code(proto.Enum):
        r"""Code describes the state of a Membership resource.

        Values:
            CODE_UNSPECIFIED (0):
                The code is not set.
            CREATING (1):
                The cluster is being registered.
            READY (2):
                The cluster is registered.
            DELETING (3):
                The cluster is being unregistered.
            UPDATING (4):
                The Membership is being updated.
            SERVICE_UPDATING (5):
                The Membership is being updated by the Hub
                Service.
        """

        CODE_UNSPECIFIED = 0
        CREATING = 1
        READY = 2
        DELETING = 3
        UPDATING = 4
        SERVICE_UPDATING = 5

    code: Code = proto.Field(
        proto.ENUM,
        number=1,
        enum=Code,
    )


class Authority(proto.Message):
    r"""Authority encodes how Google will recognize identities from
    this Membership. See the workload identity documentation for
    more details:

    https://cloud.google.com/kubernetes-engine/docs/how-to/workload-identity

    Attributes:
        issuer (str):
            Optional. A JSON Web Token (JWT) issuer URI. ``issuer`` must
            start with ``https://`` and be a valid URL with length <2000
            characters.

            If set, then Google will allow valid OIDC tokens from this
            issuer to authenticate within the workload_identity_pool.
            OIDC discovery will be performed on this URI to validate
            tokens from the issuer.

            Clearing ``issuer`` disables Workload Identity. ``issuer``
            cannot be directly modified; it must be cleared (and
            Workload Identity disabled) before using a new issuer (and
            re-enabling Workload Identity).
        workload_identity_pool (str):
            Output only. The name of the workload identity pool in which
            ``issuer`` will be recognized.

            There is a single Workload Identity Pool per Hub that is
            shared between all Memberships that belong to that Hub. For
            a Hub hosted in {PROJECT_ID}, the workload pool format is
            ``{PROJECT_ID}.hub.id.goog``, although this is subject to
            change in newer versions of this API.
        identity_provider (str):
            Output only. An identity provider that reflects the
            ``issuer`` in the workload identity pool.
        oidc_jwks (bytes):
            Optional. OIDC verification keys for this Membership in JWKS
            format (RFC 7517).

            When this field is set, OIDC discovery will NOT be performed
            on ``issuer``, and instead OIDC tokens will be validated
            using this field.
    """

    issuer: str = proto.Field(
        proto.STRING,
        number=1,
    )
    workload_identity_pool: str = proto.Field(
        proto.STRING,
        number=2,
    )
    identity_provider: str = proto.Field(
        proto.STRING,
        number=3,
    )
    oidc_jwks: bytes = proto.Field(
        proto.BYTES,
        number=4,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from .membership import (
    ApplianceCluster,
    Authority,
    ConnectAgent,
    ConnectAgentResource,
    CreateMembershipRequest,
    DeleteMembershipRequest,
    EdgeCluster,
    GenerateConnectManifestRequest,
    GenerateConnectManifestResponse,
    GenerateExclusivityManifestRequest,
    GenerateExclusivityManifestResponse,
    GetMembershipRequest,
    GkeCluster,
    KubernetesMetadata,
    KubernetesResource,
    ListMembershipsRequest,
    ListMembershipsResponse,
    Membership,
    MembershipEndpoint,
    MembershipState,
    MonitoringConfig,
    MultiCloudCluster,
    OnPremCluster,
    OperationMetadata,
    ResourceManifest,
    ResourceOptions,
    TypeMeta,
    UpdateMembershipRequest,
    ValidateExclusivityRequest,
    ValidateExclusivityResponse,
)

__all__ = (
    "ApplianceCluster",
    "Authority",
    "ConnectAgent",
    "ConnectAgentResource",
    "CreateMembershipRequest",
    "DeleteMembershipRequest",
    "EdgeCluster",
    "GenerateConnectManifestRequest",
    "GenerateConnectManifestResponse",
    "GenerateExclusivityManifestRequest",
    "GenerateExclusivityManifestResponse",
    "GetMembershipRequest",
    "GkeCluster",
    "KubernetesMetadata",
    "KubernetesResource",
    "ListMembershipsRequest",
    "ListMembershipsResponse",
    "Membership",
    "MembershipEndpoint",
    "MembershipState",
    "MonitoringConfig",
    "MultiCloudCluster",
    "OnPremCluster",
    "OperationMetadata",
    "ResourceManifest",
    "ResourceOptions",
    "TypeMeta",
    "UpdateMembershipRequest",
    "ValidateExclusivityRequest",
    "ValidateExclusivityResponse",
)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import google.protobuf.field_mask_pb2 as field_mask_pb2  # type: ignore
import google.protobuf.timestamp_pb2 as timestamp_pb2  # type: ignore
import google.rpc.status_pb2 as status_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.gkehub.v1beta1",
    manifest={
        "Membership",
        "MembershipEndpoint",
        "KubernetesResource",
        "ResourceOptions",
        "ResourceManifest",
        "GkeCluster",
        "OnPremCluster",
        "MultiCloudCluster",
        "EdgeCluster",
        "ApplianceCluster",
        "KubernetesMetadata",
        "Authority",
        "MonitoringConfig",
        "MembershipState",
        "ListMembershipsRequest",
        "ListMembershipsResponse",
        "GetMembershipRequest",
        "CreateMembershipRequest",
        "DeleteMembershipRequest",
        "UpdateMembershipRequest",
        "GenerateConnectManifestRequest",
        "GenerateConnectManifestResponse",
        "ConnectAgentResource",
        "TypeMeta",
        "ConnectAgent",
        "ValidateExclusivityRequest",
        "ValidateExclusivityResponse",
        "GenerateExclusivityManifestRequest",
        "GenerateExclusivityManifestResponse",
        "OperationMetadata",
    },
)


class Membership(proto.Message):
    r"""Membership contains information about a member cluster.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        name (str):
            Output only. The full, unique name of this Membership
            resource in the format
            ``projects/*/locations/*/memberships/{membership_id}``, set
            during creation.

            ``membership_id`` must be a valid RFC 1123 compliant DNS
            label:

            1. At most 63 characters in length
            2. It must consist of lower case alphanumeric characters or
               ``-``
            3. It must start and end with an alphanumeric character

            Which can be expressed as the regex:
            ``[a-z0-9]([-a-z0-9]*[a-z0-9])?``, with a maximum length of
            63 characters.
        labels (MutableMapping[str, str]):
            Optional. GCP labels for this membership.
        description (str):
            Optional. Description of this membership, limited to 63
            characters. Must match the regex:
            ``[a-zA-Z0-9][a-zA-Z0-9_\-\.\ ]*``
        endpoint (google.cloud.gkehub_v1beta1.types.MembershipEndpoint):
            Optional. Endpoint information to reach this
            member.

            This field is a member of `oneof`_ ``type``.
        state (google.cloud.gkehub_v1beta1.types.MembershipState):
            Output only. State of the Membership
            resource.
        authority (google.cloud.gkehub_v1beta1.types.Authority):
            Optional. How to identify workloads from this
            Membership. See the documentation on Workload
            Identity for more details:

            https://cloud.google.com/kubernetes-engine/docs/how-to/workload-identity
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. When the Membership was created.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. When the Membership was last
            updated.
        delete_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. When the Membership was deleted.
        external_id (str):
            Optional. An externally-generated and managed ID for this
            Membership. This ID may be modified after creation, but this
            is not recommended. For GKE clusters, external_id is managed
            by the Hub API and updates will be ignored.

            The ID must match the regex:
            ``[a-zA-Z0-9][a-zA-Z0-9_\-\.]*``

            If this Membership represents a Kubernetes cluster, this
            value should be set to the UID of the ``kube-system``
            namespace object.
        last_connection_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. For clusters using Connect, the
            timestamp of the most recent connection
            established with Google Cloud. This time is
            updated every several minutes, not continuously.
            For clusters that do not use GKE Connect, or
            that have never connected successfully, this
            field will be unset.
        unique_id (str):
            Output only. Google-generated UUID for this resource. This
            is unique across all Membership resources. If a Membership
            resource is deleted and another resource with the same name
            is created, it gets a different unique_id.
        infrastructure_type (google.cloud.gkehub_v1beta1.types.Membership.InfrastructureType):
            Optional. The infrastructure type this
            Membership is running on.
        monitoring_config (google.cloud.gkehub_v1beta1.types.MonitoringConfig):
            Optional. The monitoring config information
            for this membership.
    """

    class InfrastructureType(proto.Enum):
        r"""Specifies the infrastructure type of a Membership.
        Infrastructure type is used by Hub to control
        infrastructure-specific behavior, including pricing.

        Each GKE distribution (on-GCP, on-Prem, on-X,...) will set this
        field automatically, but Attached Clusters customers should
        specify a type during registration.

        Values:
            INFRASTRUCTURE_TYPE_UNSPECIFIED (0):
                No type was specified. Some Hub functionality
                may require a type be specified, and will not
                support Memberships with this value.
            ON_PREM (1):
                Private infrastructure that is owned or
                operated by customer. This includes GKE
                distributions such as GKE-OnPrem and
                GKE-OnBareMetal.
            MULTI_CLOUD (2):
                Public cloud infrastructure.
        """

        INFRASTRUCTURE_TYPE_UNSPECIFIED = 0
        ON_PREM = 1
        MULTI_CLOUD = 2

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=2,
    )
    description: str = proto.Field(
        proto.STRING,
        number=3,
    )
    endpoint: "MembershipEndpoint" = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="type",
        message="MembershipEndpoint",
    )
    state: "MembershipState" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="MembershipState",
    )
    authority: "Authority" = proto.Field(
        proto.MESSAGE,
        number=9,
        message="Authority",
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=6,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=7,
        message=timestamp_pb2.Timestamp,
    )
    delete_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=8,
        message=timestamp_pb2.Timestamp,
    )
    external_id: str = proto.Field(
        proto.STRING,
        number=10,
    )
    last_connection_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=11,
        message=timestamp_pb2.Timestamp,
    )
    unique_id: str = proto.Field(
        proto.STRING,
        number=12,
    )
    infrastructure_type: InfrastructureType = proto.Field(
        proto.ENUM,
        number=13,
        enum=InfrastructureType,
    )
    monitoring_config: "MonitoringConfig" = proto.Field(
        proto.MESSAGE,
        number=14,
        message="MonitoringConfig",
    )


class MembershipEndpoint(proto.Message):
    r"""MembershipEndpoint contains information needed to contact a
    Kubernetes API, endpoint and any additional Kubernetes metadata.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gke_cluster (google.cloud.gkehub_v1beta1.types.GkeCluster):
            Optional. Specific information for a
            GKE-on-GCP cluster.

            This field is a member of `oneof`_ ``type``.
        on_prem_cluster (google.cloud.gkehub_v1beta1.types.OnPremCluster):
            Optional. Specific information for a GKE
            On-Prem cluster. An onprem user-cluster who has
            no resourceLink is not allowed to use this
            field, it should have a nil "type" instead.

            This field is a member of `oneof`_ ``type``.
        multi_cloud_cluster (google.cloud.gkehub_v1beta1.types.MultiCloudCluster):
            Optional. Specific information for a GKE
            Multi-Cloud cluster.

            This field is a member of `oneof`_ ``type``.
        edge_cluster (google.cloud.gkehub_v1beta1.types.EdgeCluster):
            Optional. Specific information for a Google
            Edge cluster.

            This field is a member of `oneof`_ ``type``.
        appliance_cluster (google.cloud.gkehub_v1beta1.types.ApplianceCluster):
            Optional. Specific information for a GDC Edge
            Appliance cluster.

            This field is a member of `oneof`_ ``type``.
        kubernetes_metadata (google.cloud.gkehub_v1beta1.types.KubernetesMetadata):
            Output only. Useful Kubernetes-specific
            metadata.
        kubernetes_resource (google.cloud.gkehub_v1beta1.types.KubernetesResource):
            Optional. The in-cluster Kubernetes Resources that should be
            applied for a correctly registered cluster, in the steady
            state. These resources:

            - Ensure that the cluster is exclusively registered to one
              and only one Hub Membership.
            - Propagate Workload Pool Information available in the
              Membership Authority field.
            - Ensure proper initial configuration of default Hub
              Features.
    """

    gke_cluster: "GkeCluster" = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="type",
        message="GkeCluster",
    )
    on_prem_cluster: "OnPremCluster" = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="type",
        message="OnPremCluster",
    )
    multi_cloud_cluster: "MultiCloudCluster" = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="type",
        message="MultiCloudCluster",
    )
    edge_cluster: "EdgeCluster" = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="type",
        message="EdgeCluster",
    )
    appliance_cluster: "ApplianceCluster" = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof="type",
        message="ApplianceCluster",
    )
    kubernetes_metadata: "KubernetesMetadata" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="KubernetesMetadata",
    )
    kubernetes_resource: "KubernetesResource" = proto.Field(
        proto.MESSAGE,
        number=6,
        message="KubernetesResource",
    )


class KubernetesResource(proto.Message):
    r"""KubernetesResource contains the YAML manifests and
    configuration for Membership Kubernetes resources in the
    cluster. After CreateMembership or UpdateMembership, these
    resources should be re-applied in the cluster.

    Attributes:
        membership_cr_manifest (str):
            Input only. The YAML representation of the
            Membership CR. This field is ignored for GKE
            clusters where Hub can read the CR directly.

            Callers should provide the CR that is currently
            present in the cluster during CreateMembership
            or UpdateMembership, or leave this field empty
            if none exists. The CR manifest is used to
            validate the cluster has not been registered
            with another Membership.
        membership_resources (MutableSequence[google.cloud.gkehub_v1beta1.types.ResourceManifest]):
            Output only. Additional Kubernetes resources
            that need to be applied to the cluster after
            Membership creation, and after every update.

            This field is only populated in the Membership
            returned from a successful long-running
            operation from CreateMembership or
            UpdateMembership. It is not populated during
            normal GetMembership or ListMemberships
            requests. To get the resource manifest after the
            initial registration, the caller should make a
            UpdateMembership call with an empty field mask.
        connect_resources (MutableSequence[google.cloud.gkehub_v1beta1.types.ResourceManifest]):
            Output only. The Kubernetes resources for
            installing the GKE Connect agent
            This field is only populated in the Membership
            returned from a successful long-running
            operation from CreateMembership or
            UpdateMembership. It is not populated during
            normal GetMembership or ListMemberships
            requests. To get the resource manifest after the
            initial registration, the caller should make a
            UpdateMembership call with an empty field mask.
        resource_options (google.cloud.gkehub_v1beta1.types.ResourceOptions):
            Optional. Options for Kubernetes resource
            generation.
    """

    membership_cr_manifest: str = proto.Field(
        proto.STRING,
        number=1,
    )
    membership_resources: MutableSequence["ResourceManifest"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="ResourceManifest",
    )
    connect_resources: MutableSequence["ResourceManifest"] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="ResourceManifest",
    )
    resource_options: "ResourceOptions" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="ResourceOptions",
    )


class ResourceOptions(proto.Message):
    r"""ResourceOptions represent options for Kubernetes resource
    generation.

    Attributes:
        connect_version (str):
            Optional. The Connect agent version to use for
            connect_resources. Defaults to the latest GKE Connect
            version. The version must be a currently supported version,
            obsolete versions will be rejected.
        v1beta1_crd (bool):
            Optional. Use ``apiextensions/v1beta1`` instead of
            ``apiextensions/v1`` for CustomResourceDefinition resources.
            This option should be set for clusters with Kubernetes
            apiserver versions <1.16.
        k8s_version (str):
            Optional. Major and minor version of the Kubernetes cluster.
            This is only used to determine which version to use for the
            CustomResourceDefinition resources,
            ``apiextensions/v1beta1`` or\ ``apiextensions/v1``.
        k8s_git_version (str):
            Optional. Git version of the Kubernetes
            cluster. This is only used to gate the Connect
            Agent migration to svc.id.goog on GDC-SO
            1.33.100 patch and above.
    """

    connect_version: str = proto.Field(
        proto.STRING,
        number=1,
    )
    v1beta1_crd: bool = proto.Field(
        proto.BOOL,
        number=2,
    )
    k8s_version: str = proto.Field(
        proto.STRING,
        number=3,
    )
    k8s_git_version: str = proto.Field(
        proto.STRING,
        number=4,
    )


class ResourceManifest(proto.Message):
    r"""ResourceManifest represents a single Kubernetes resource to
    be applied to the cluster.

    Attributes:
        manifest (str):
            YAML manifest of the resource.
        cluster_scoped (bool):
            Whether the resource provided in the manifest is
            ``cluster_scoped``. If unset, the manifest is assumed to be
            namespace scoped.

            This field is used for REST mapping when applying the
            resource in a cluster.
    """

    manifest: str = proto.Field(
        proto.STRING,
        number=1,
    )
    cluster_scoped: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class GkeCluster(proto.Message):
    r"""GkeCluster contains information specific to GKE clusters.

    Attributes:
        resource_link (str):
            Immutable. Self-link of the GCP resource for
            the GKE cluster. For example:
            //container.googleapis.com/projects/my-project/locations/us-west1-a/clusters/my-cluster

            Zonal clusters are also supported.
        cluster_missing (bool):
            Output only. If cluster_missing is set then it denotes that
            the GKE cluster no longer exists in the GKE Control Plane.
    """

    resource_link: str = proto.Field(
        proto.STRING,
        number=1,
    )
    cluster_missing: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class OnPremCluster(proto.Message):
    r"""OnPremCluster contains information specific to GKE On-Prem
    clusters.

    Attributes:
        resource_link (str):
            Immutable. Self-link of the GCP resource for
            the GKE On-Prem cluster. For example:

            //gkeonprem.googleapis.com/projects/my-project/locations/us-west1-a/vmwareClusters/my-cluster
            //gkeonprem.googleapis.com/projects/my-project/locations/us-west1-a/bareMetalClusters/my-cluster
        cluster_missing (bool):
            Output only. If cluster_missing is set then it denotes that
            API(gkeonprem.googleapis.com) resource for this GKE On-Prem
            cluster no longer exists.
        admin_cluster (bool):
            Immutable. Whether the cluster is an admin
            cluster.
        cluster_type (google.cloud.gkehub_v1beta1.types.OnPremCluster.ClusterType):
            Immutable. The on prem cluster's type.
    """

    class ClusterType(proto.Enum):
        r"""ClusterType describes on prem cluster's type.

        Values:
            CLUSTERTYPE_UNSPECIFIED (0):
                The ClusterType is not set.
            BOOTSTRAP (1):
                The ClusterType is bootstrap cluster.
            HYBRID (2):
                The ClusterType is baremetal hybrid cluster.
            STANDALONE (3):
                The ClusterType is baremetal standalone
                cluster.
            USER (4):
                The ClusterType is user cluster.
        """

        CLUSTERTYPE_UNSPECIFIED = 0
        BOOTSTRAP = 1
        HYBRID = 2
        STANDALONE = 3
        USER = 4

    resource_link: str = proto.Field(
        proto.STRING,
        number=1,
    )
    cluster_missing: bool = proto.Field(
        proto.BOOL,
        number=2,
    )
    admin_cluster: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    cluster_type: ClusterType = proto.Field(
        proto.ENUM,
        number=4,
        enum=ClusterType,
    )


class MultiCloudCluster(proto.Message):
    r"""MultiCloudCluster contains information specific to GKE
    Multi-Cloud clusters.

    Attributes:
        resource_link (str):
            Immutable. Self-link of the GCP resource for
            the GKE Multi-Cloud cluster. For example:

            //gkemulticloud.googleapis.com/projects/my-project/locations/us-west1-a/awsClusters/my-cluster
            //gkemulticloud.googleapis.com/projects/my-project/locations/us-west1-a/azureClusters/my-cluster
            //gkemulticloud.googleapis.com/projects/my-project/locations/us-west1-a/attachedClusters/my-cluster
        cluster_missing (bool):
            Output only. If cluster_missing is set then it denotes that
            API(gkemulticloud.googleapis.com) resource for this GKE
            Multi-Cloud cluster no longer exists.
    """

    resource_link: str = proto.Field(
        proto.STRING,
        number=1,
    )
    cluster_missing: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class EdgeCluster(proto.Message):
    r"""EdgeCluster contains information specific to Google Edge
    Clusters.

    Attributes:
        resource_link (str):
            Immutable. Self-link of the GCP resource for
            the Edge Cluster. For example:

            //edgecontainer.googleapis.com/projects/my-project/locations/us-west1-a/clusters/my-cluster
    """

    resource_link: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ApplianceCluster(proto.Message):
    r"""ApplianceCluster contains information specific to GDC Edge
    Appliance Clusters.

    Attributes:
        resource_link (str):
            Immutable. Self-link of the GCP resource for
            the Appliance Cluster. For example:

            //transferappliance.googleapis.com/projects/my-project/locations/us-west1-a/appliances/my-appliance
    """

    resource_link: str = proto.Field(
        proto.STRING,
        number=1,
    )


class KubernetesMetadata(proto.Message):
    r"""KubernetesMetadata provides informational metadata for
    Memberships representing Kubernetes clusters.

    Attributes:
        kubernetes_api_server_version (str):
            Output only. Kubernetes API server version
            string as reported by '/version'.
        node_provider_id (str):
            Output only. Node providerID as reported by the first node
            in the list of nodes on the Kubernetes endpoint. On
            Kubernetes platforms that support zero-node clusters (like
            GKE-on-GCP), the node_count will be zero and the
            node_provider_id will be empty.
        node_count (int):
            Output only. Node count as reported by
            Kubernetes nodes resources.
        vcpu_count (int):
            Output only. vCPU count as reported by
            Kubernetes nodes resources.
        memory_mb (int):
            Output only. The total memory capacity as
            reported by the sum of all Kubernetes nodes
            resources, defined in MB.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time at which these details were last
            updated. This update_time is different from the
            Membership-level update_time since EndpointDetails are
            updated internally for API consumers.
    """

    kubernetes_api_server_version: str = proto.Field(
        proto.STRING,
        number=1,
    )
    node_provider_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    node_count: int = proto.Field(
        proto.INT32,
        number=3,
    )
    vcpu_count: int = proto.Field(
        proto.INT32,
        number=4,
    )
    memory_mb: int = proto.Field(
        proto.INT32,
        number=5,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=100,
        message=timestamp_pb2.Timestamp,
    )


class Authority(proto.Message):
    r"""Authority encodes how Google will recognize identities from
    this Membership. See the workload identity documentation for
    more details:

    https://cloud.google.com/kubernetes-engine/docs/how-to/workload-identity

    Attributes:
        issuer (str):
            Optional. A JSON Web Token (JWT) issuer URI. ``issuer`` must
            start with ``https://`` and be a valid URL with length <2000
            characters.

            If set, then Google will allow valid OIDC tokens from this
            issuer to authenticate within the workload_identity_pool.
            OIDC discovery will be performed on this URI to validate
            tokens from the issuer.

            Clearing ``issuer`` disables Workload Identity. ``issuer``
            cannot be directly modified; it must be cleared (and
            Workload Identity disabled) before using a new issuer (and
            re-enabling Workload Identity).
        workload_identity_pool (str):
            Output only. The name of the workload identity pool in which
            ``issuer`` will be recognized.

            There is a single Workload Identity Pool per Hub that is
            shared between all Memberships that belong to that Hub. For
            a Hub hosted in {PROJECT_ID}, the workload pool format is
            ``{PROJECT_ID}.hub.id.goog``, although this is subject to
            change in newer versions of this API.
        identity_provider (str):
            Output only. An identity provider that reflects the
            ``issuer`` in the workload identity pool.
        oidc_jwks (bytes):
            Optional. OIDC verification keys for this Membership in JWKS
            format (RFC 7517).

            When this field is set, OIDC discovery will NOT be performed
            on ``issuer``, and instead OIDC tokens will be validated
            using this field.
    """

    issuer: str = proto.Field(
        proto.STRING,
        number=1,
    )
    workload_identity_pool: str = proto.Field(
        proto.STRING,
        number=2,
    )
    identity_provider: str = proto.Field(
        proto.STRING,
        number=3,
    )
    oidc_jwks: bytes = proto.Field(
        proto.BYTES,
        number=4,
    )


class MonitoringConfig(proto.Message):
    r"""This field informs Fleet-based applications/services/UIs with
    the necessary information for where each underlying Cluster
    reports its metrics.

    Attributes:
        project_id (str):
            Immutable. Project used to report Metrics
        location (str):
            Immutable. Location used to report Metrics
        cluster (str):
            Immutable. Cluster name used to report metrics. For Anthos
            on VMWare/Baremetal, it would be in format
            ``memberClusters/cluster_name``; And for Anthos on
            MultiCloud, it would be in format
            ``{azureClusters, awsClusters}/cluster_name``.
        kubernetes_metrics_prefix (str):
            Kubernetes system metrics, if available, are
            written to this prefix. This defaults to
            kubernetes.io for GKE, and kubernetes.io/anthos
            for Anthos eventually. Noted: Anthos MultiCloud
            will have kubernetes.io prefix today but will
            migration to be under kubernetes.io/anthos
        cluster_hash (str):
            Immutable. Cluster hash, this is a unique
            string generated by google code, which does not
            contain any PII, which we can use to reference
            the cluster. This is expected to be created by
            the monitoring stack and persisted into the
            Cluster object as well as to GKE-Hub.
    """

    project_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    location: str = proto.Field(
        proto.STRING,
        number=2,
    )
    cluster: str = proto.Field(
        proto.STRING,
        number=3,
    )
    kubernetes_metrics_prefix: str = proto.Field(
        proto.STRING,
        number=4,
    )
    cluster_hash: str = proto.Field(
        proto.STRING,
        number=5,
    )


class MembershipState(proto.Message):
    r"""State of the Membership resource.

    Attributes:
        code (google.cloud.gkehub_v1beta1.types.MembershipState.Code):
            Output only. The current state of the
            Membership resource.
        description (str):
            This field is never set by the Hub Service.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            This field is never set by the Hub Service.
    """

    class Code(proto.Enum):
        r"""Code describes the state of a Membership resource.

        Values:
            CODE_UNSPECIFIED (0):
                The code is not set.
            CREATING (1):
                The cluster is being registered.
            READY (2):
                The cluster is registered.
            DELETING (3):
                The cluster is being unregistered.
            UPDATING (4):
                The Membership is being updated.
            SERVICE_UPDATING (5):
                The Membership is being updated by the Hub
                Service.
        """

        CODE_UNSPECIFIED = 0
        CREATING = 1
        READY = 2
        DELETING = 3
        UPDATING = 4
        SERVICE_UPDATING = 5

    code: Code = proto.Field(
        proto.ENUM,
        number=1,
        enum=Code,
    )
    description: str = proto.Field(
        proto.STRING,
        number=2,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class ListMembershipsRequest(proto.Message):
    r"""Request message for ``GkeHubMembershipService.ListMemberships``
    method.

    Attributes:
        parent (str):
            Required. The parent (project and location) where the
            Memberships will be listed. Specified in the format
            ``projects/*/locations/*``. ``projects/*/locations/-`` list
            memberships in all the regions.
        page_size (int):
            Optional. When requesting a 'page' of resources,
            ``page_size`` specifies number of resources to return. If
            unspecified or set to 0, all resources will be returned.
        page_token (str):
            Optional. Token returned by previous call to
            ``ListMemberships`` which specifies the position in the list
            from where to continue listing the resources.
        filter (str):
            Optional. Lists Memberships that match the filter
            expression, following the syntax outlined in
            https://google.aip.dev/160.

            Examples:

            - Name is ``bar`` in project ``foo-proj`` and location
              ``global``:

              name = "projects/foo-proj/locations/global/membership/bar"

            - Memberships that have a label called ``foo``:

              labels.foo:\*

            - Memberships that have a label called ``foo`` whose value
              is ``bar``:

              labels.foo = bar

            - Memberships in the CREATING state:

              state = CREATING
        order_by (str):
            Optional. One or more fields to compare and
            use to sort the output. See
            https://google.aip.dev/132#ordering.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=4,
    )
    order_by: str = proto.Field(
        proto.STRING,
        number=5,
    )


class ListMembershipsResponse(proto.Message):
    r"""Response message for the ``GkeHubMembershipService.ListMemberships``
    method.

    Attributes:
        resources (MutableSequence[google.cloud.gkehub_v1beta1.types.Membership]):
            The list of matching Memberships.
        next_page_token (str):
            A token to request the next page of resources from the
            ``ListMemberships`` method. The value of an empty string
            means that there are no more resources to return.
        unreachable (MutableSequence[str]):
            List of locations that could not be reached
            while fetching this list.
    """

    @property
    def raw_page(self):
        return self

    resources: MutableSequence["Membership"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="Membership",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    unreachable: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )


class GetMembershipRequest(proto.Message):
    r"""Request message for ``GkeHubMembershipService.GetMembership``
    method.

    Attributes:
        name (str):
            Required. The Membership resource name in the format
            ``projects/*/locations/*/memberships/*``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class CreateMembershipRequest(proto.Message):
    r"""Request message for the ``GkeHubMembershipService.CreateMembership``
    method.

    Attributes:
        parent (str):
            Required. The parent (project and location) where the
            Memberships will be created. Specified in the format
            ``projects/*/locations/*``.
        membership_id (str):
            Required. Client chosen ID for the membership.
            ``membership_id`` must be a valid RFC 1123 compliant DNS
            label:

            1. At most 63 characters in length
            2. It must consist of lower case alphanumeric characters or
               ``-``
            3. It must start and end with an alphanumeric character

            Which can be expressed as the regex:
            ``[a-z0-9]([-a-z0-9]*[a-z0-9])?``, with a maximum length of
            63 characters.
        resource (google.cloud.gkehub_v1beta1.types.Membership):
            Required. The membership to create.
        request_id (str):
            Optional. A request ID to identify requests.
            Specify a unique request ID so that if you must
            retry your request, the server will know to
            ignore the request if it has already been
            completed. The server will guarantee that for at
            least 60 minutes after the first request.

            For example, consider a situation where you make
            an initial request and the request times out. If
            you make the request again with the same request
            ID, the server can check if original operation
            with the same request ID was received, and if
            so, will ignore the second request. This
            prevents clients from accidentally creating
            duplicate commitments.

            The request ID must be a valid UUID with the
            exception that zero UUID is not supported
            (00000000-0000-0000-0000-000000000000).
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    membership_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    resource: "Membership" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="Membership",
    )
    request_id: str = proto.Field(
        proto.STRING,
        number=4,
    )


class DeleteMembershipRequest(proto.Message):
    r"""Request message for ``GkeHubMembershipService.DeleteMembership``
    method.

    Attributes:
        name (str):
            Required. The Membership resource name in the format
            ``projects/*/locations/*/memberships/*``.
        request_id (str):
            Optional. A request ID to identify requests.
            Specify a unique request ID so that if you must
            retry your request, the server will know to
            ignore the request if it has already been
            completed. The server will guarantee that for at
            least 60 minutes after the first request.

            For example, consider a situation where you make
            an initial request and the request times out. If
            you make the request again with the same request
            ID, the server can check if original operation
            with the same request ID was received, and if
            so, will ignore the second request. This
            prevents clients from accidentally creating
            duplicate commitments.

            The request ID must be a valid UUID with the
            exception that zero UUID is not supported
            (00000000-0000-0000-0000-000000000000).
        force (bool):
            Optional. If set to true, any subresource
            from this Membership will also be deleted.
            Otherwise, the request will only work if the
            Membership has no subresource.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    request_id: str = proto.Field(
        proto.STRING,
        number=4,
    )
    force: bool = proto.Field(
        proto.BOOL,
        number=5,
    )


class UpdateMembershipRequest(proto.Message):
    r"""Request message for ``GkeHubMembershipService.UpdateMembership``
    method.

    Attributes:
        name (str):
            Required. The membership resource name in the format:
            ``projects/[project_id]/locations/global/memberships/[membership_id]``
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. Mask of fields to update. At least
            one field path must be specified in this mask.
        resource (google.cloud.gkehub_v1beta1.types.Membership):
            Required. Only fields specified in update_mask are updated.
            If you specify a field in the update_mask but don't specify
            its value here that field will be deleted. If you are
            updating a map field, set the value of a key to null or
            empty string to delete the key from the map. It's not
            possible to update a key's value to the empty string. If you
            specify the update_mask to be a special path "\*", fully
            replaces all user-modifiable fields to match ``resource``.
        request_id (str):
            Optional. A request ID to identify requests.
            Specify a unique request ID so that if you must
            retry your request, the server will know to
            ignore the request if it has already been
            completed. The server will guarantee that for at
            least 60 minutes after the first request.

            For example, consider a situation where you make
            an initial request and the request times out. If
            you make the request again with the same request
            ID, the server can check if original operation
            with the same request ID was received, and if
            so, will ignore the second request. This
            prevents clients from accidentally creating
            duplicate commitments.

            The request ID must be a valid UUID with the
            exception that zero UUID is not supported
            (00000000-0000-0000-0000-000000000000).
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )
    resource: "Membership" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="Membership",
    )
    request_id: str = proto.Field(
        proto.STRING,
        number=4,
    )


class GenerateConnectManifestRequest(proto.Message):
    r"""Request message for
    ``GkeHubMembershipService.GenerateConnectManifest`` method.

    Attributes:
        name (str):
            Required. The Membership resource name the Agent will
            associate with, in the format
            ``projects/*/locations/*/memberships/*``.
        connect_agent (google.cloud.gkehub_v1beta1.types.ConnectAgent):
            Optional. The connect agent to generate
            manifest for.
        version (str):
            Optional. The Connect agent version to use.
            Defaults to the most current version.
        is_upgrade (bool):
            Optional. If true, generate the resources for
            upgrade only. Some resources generated only for
            installation (e.g. secrets) will be excluded.
        registry (str):
            Optional. The registry to fetch the connect
            agent image from. Defaults to gcr.io/gkeconnect.
        image_pull_secret_content (bytes):
            Optional. The image pull secret content for
            the registry, if not public.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    connect_agent: "ConnectAgent" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="ConnectAgent",
    )
    version: str = proto.Field(
        proto.STRING,
        number=3,
    )
    is_upgrade: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    registry: str = proto.Field(
        proto.STRING,
        number=5,
    )
    image_pull_secret_content: bytes = proto.Field(
        proto.BYTES,
        number=6,
    )


class GenerateConnectManifestResponse(proto.Message):
    r"""GenerateConnectManifestResponse contains manifest information
    for installing/upgrading a Connect agent.

    Attributes:
        manifest (MutableSequence[google.cloud.gkehub_v1beta1.types.ConnectAgentResource]):
            The ordered list of Kubernetes resources that
            need to be applied to the cluster for GKE
            Connect agent installation/upgrade.
    """

    manifest: MutableSequence["ConnectAgentResource"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="ConnectAgentResource",
    )


class ConnectAgentResource(proto.Message):
    r"""ConnectAgentResource represents a Kubernetes resource
    manifest for Connect Agent deployment.

    Attributes:
        type_ (google.cloud.gkehub_v1beta1.types.TypeMeta):
            Kubernetes type of the resource.
        manifest (str):
            YAML manifest of the resource.
    """

    type_: "TypeMeta" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="TypeMeta",
    )
    manifest: str = proto.Field(
        proto.STRING,
        number=2,
    )


class TypeMeta(proto.Message):
    r"""TypeMeta is the type information needed for content
    unmarshalling of Kubernetes resources in the manifest.

    Attributes:
        kind (str):
            Kind of the resource (e.g. Deployment).
        api_version (str):
            APIVersion of the resource (e.g. v1).
    """

    kind: str = proto.Field(
        proto.STRING,
        number=1,
    )
    api_version: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ConnectAgent(proto.Message):
    r"""The information required from end users to use GKE Connect.

    Attributes:
        name (str):
            Do not set.
        proxy (bytes):
            Optional. URI of a proxy if connectivity from the agent to
            gkeconnect.googleapis.com requires the use of a proxy.
            Format must be in the form ``http(s)://{proxy_address}``,
            depending on the HTTP/HTTPS protocol supported by the proxy.
            This will direct the connect agent's outbound traffic
            through a HTTP(S) proxy.
        namespace (str):
            Optional. Namespace for GKE Connect agent resources.
            Defaults to ``gke-connect``.

            The Connect Agent is authorized automatically when run in
            the default namespace. Otherwise, explicit authorization
            must be granted with an additional IAM binding.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    proxy: bytes = proto.Field(
        proto.BYTES,
        number=2,
    )
    namespace: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ValidateExclusivityRequest(proto.Message):
    r"""The request to validate the existing state of the membership
    CR in the cluster.

    Attributes:
        parent (str):
            Required. The parent (project and location) where the
            Memberships will be created. Specified in the format
            ``projects/*/locations/*``.
        cr_manifest (str):
            Optional. The YAML of the membership CR in
            the cluster. Empty if the membership CR does not
            exist.
        intended_membership (str):
            Required. The intended membership name under the ``parent``.
            This method only does validation in anticipation of a
            CreateMembership call with the same name.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    cr_manifest: str = proto.Field(
        proto.STRING,
        number=2,
    )
    intended_membership: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ValidateExclusivityResponse(proto.Message):
    r"""The response of exclusivity artifacts validation result
    status.

    Attributes:
        status (google.rpc.status_pb2.Status):
            The validation result.

            - ``OK`` means that exclusivity is validated, assuming the
              manifest produced by GenerateExclusivityManifest is
              successfully applied.
            - ``ALREADY_EXISTS`` means that the Membership CRD is
              already owned by another Hub. See ``status.message`` for
              more information.
    """

    status: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=1,
        message=status_pb2.Status,
    )


class GenerateExclusivityManifestRequest(proto.Message):
    r"""The request to generate the manifests for exclusivity
    artifacts.

    Attributes:
        name (str):
            Required. The Membership resource name in the format
            ``projects/*/locations/*/memberships/*``.
        crd_manifest (str):
            Optional. The YAML manifest of the membership CRD retrieved
            by ``kubectl get customresourcedefinitions membership``.
            Leave empty if the resource does not exist.
        cr_manifest (str):
            Optional. The YAML manifest of the membership CR retrieved
            by ``kubectl get memberships membership``. Leave empty if
            the resource does not exist.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    crd_manifest: str = proto.Field(
        proto.STRING,
        number=2,
    )
    cr_manifest: str = proto.Field(
        proto.STRING,
        number=3,
    )


class GenerateExclusivityManifestResponse(proto.Message):
    r"""The response of the exclusivity artifacts manifests for the
    client to apply.

    Attributes:
        crd_manifest (str):
            The YAML manifest of the membership CRD to
            apply if a newer version of the CRD is
            available. Empty if no update needs to be
            applied.
        cr_manifest (str):
            The YAML manifest of the membership CR to
            apply if a new version of the CR is available.
            Empty if no update needs to be applied.
    """

    crd_manifest: str = proto.Field(
        proto.STRING,
        number=1,
    )
    cr_manifest: str = proto.Field(
        proto.STRING,
        number=2,
    )


class OperationMetadata(proto.Message):
    r"""Represents the metadata of the long-running operation.

    Attributes:
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time the operation was
            created.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time the operation finished
            running.
        target (str):
            Output only. Server-defined resource path for
            the target of the operation.
        verb (str):
            Output only. Name of the verb executed by the
            operation.
        status_detail (str):
            Output only. Human-readable status of the
            operation, if any.
        cancel_requested (bool):
            Output only. Identifies whether the user has requested
            cancellation of the operation. Operations that have
            successfully been cancelled have [Operation.error][] value
            with a [google.rpc.Status.code][google.rpc.Status.code] of
            1, corresponding to ``Code.CANCELLED``.
        api_version (str):
            Output only. API version used to start the
            operation.
    """

    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=1,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    target: str = proto.Field(
        proto.STRING,
        number=3,
    )
    verb: str = proto.Field(
        proto.STRING,
        number=4,
    )
    status_detail: str = proto.Field(
        proto.STRING,
        number=5,
    )
    cancel_requested: bool = proto.Field(
        proto.BOOL,
        number=6,
    )
    api_version: str = proto.Field(
        proto.STRING,
        number=7,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.gkehub.configmanagement.v1",
    manifest={
        "DeploymentState",
        "MembershipState",
        "MembershipSpec",
        "ConfigSync",
        "GitConfig",
        "OciConfig",
        "PolicyController",
        "HierarchyControllerConfig",
        "HierarchyControllerDeploymentState",
        "HierarchyControllerVersion",
        "HierarchyControllerState",
        "OperatorState",
        "InstallError",
        "ConfigSyncState",
        "ConfigSyncError",
        "ConfigSyncVersion",
        "ConfigSyncDeploymentState",
        "SyncState",
        "SyncError",
        "ErrorResource",
        "GroupVersionKind",
        "PolicyControllerState",
        "PolicyControllerVersion",
        "GatekeeperDeploymentState",
    },
)


class DeploymentState(proto.Enum):
    r"""Enum representing the state of an ACM's deployment on a
    cluster

    Values:
        DEPLOYMENT_STATE_UNSPECIFIED (0):
            Deployment's state cannot be determined
        NOT_INSTALLED (1):
            Deployment is not installed
        INSTALLED (2):
            Deployment is installed
        ERROR (3):
            Deployment was attempted to be installed, but
            has errors
        PENDING (4):
            Deployment is installing or terminating
    """

    DEPLOYMENT_STATE_UNSPECIFIED = 0
    NOT_INSTALLED = 1
    INSTALLED = 2
    ERROR = 3
    PENDING = 4


class MembershipState(proto.Message):
    r"""**Anthos Config Management**: State for a single cluster.

    Attributes:
        cluster_name (str):
            This field is set to the ``cluster_name`` field of the
            Membership Spec if it is not empty. Otherwise, it is set to
            the cluster's fleet membership name.
        membership_spec (google.cloud.gkehub.configmanagement_v1.types.MembershipSpec):
            Membership configuration in the cluster. This
            represents the actual state in the cluster,
            while the MembershipSpec in the FeatureSpec
            represents the intended state
        operator_state (google.cloud.gkehub.configmanagement_v1.types.OperatorState):
            Current install status of ACM's Operator
        config_sync_state (google.cloud.gkehub.configmanagement_v1.types.ConfigSyncState):
            Current sync status
        policy_controller_state (google.cloud.gkehub.configmanagement_v1.types.PolicyControllerState):
            PolicyController status
        hierarchy_controller_state (google.cloud.gkehub.configmanagement_v1.types.HierarchyControllerState):
            Hierarchy Controller status
    """

    cluster_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    membership_spec: "MembershipSpec" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="MembershipSpec",
    )
    operator_state: "OperatorState" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="OperatorState",
    )
    config_sync_state: "ConfigSyncState" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="ConfigSyncState",
    )
    policy_controller_state: "PolicyControllerState" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="PolicyControllerState",
    )
    hierarchy_controller_state: "HierarchyControllerState" = proto.Field(
        proto.MESSAGE,
        number=7,
        message="HierarchyControllerState",
    )


class MembershipSpec(proto.Message):
    r"""**Anthos Config Management**: Configuration for a single cluster.
    Intended to parallel the ConfigManagement CR.

    Attributes:
        config_sync (google.cloud.gkehub.configmanagement_v1.types.ConfigSync):
            Config Sync configuration for the cluster.
        policy_controller (google.cloud.gkehub.configmanagement_v1.types.PolicyController):
            Policy Controller configuration for the
            cluster.
        hierarchy_controller (google.cloud.gkehub.configmanagement_v1.types.HierarchyControllerConfig):
            Hierarchy Controller configuration for the
            cluster.
        version (str):
            Version of ACM installed.
        cluster (str):
            The user-specified cluster name used by
            Config Sync cluster-name-selector annotation or
            ClusterSelector, for applying configs to only a
            subset of clusters.
            Omit this field if the cluster's fleet
            membership name is used by Config Sync
            cluster-name-selector annotation or
            ClusterSelector. Set this field if a name
            different from the cluster's fleet membership
            name is used by Config Sync
            cluster-name-selector annotation or
            ClusterSelector.
        management (google.cloud.gkehub.configmanagement_v1.types.MembershipSpec.Management):
            Enables automatic Feature management.
    """

    class Management(proto.Enum):
        r"""Whether to automatically manage the Feature.

        Values:
            MANAGEMENT_UNSPECIFIED (0):
                Unspecified
            MANAGEMENT_AUTOMATIC (1):
                Google will manage the Feature for the
                cluster.
            MANAGEMENT_MANUAL (2):
                User will manually manage the Feature for the
                cluster.
        """

        MANAGEMENT_UNSPECIFIED = 0
        MANAGEMENT_AUTOMATIC = 1
        MANAGEMENT_MANUAL = 2

    config_sync: "ConfigSync" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="ConfigSync",
    )
    policy_controller: "PolicyController" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="PolicyController",
    )
    hierarchy_controller: "HierarchyControllerConfig" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="HierarchyControllerConfig",
    )
    version: str = proto.Field(
        proto.STRING,
        number=10,
    )
    cluster: str = proto.Field(
        proto.STRING,
        number=11,
    )
    management: Management = proto.Field(
        proto.ENUM,
        number=12,
        enum=Management,
    )


class ConfigSync(proto.Message):
    r"""Configuration for Config Sync

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        git (google.cloud.gkehub.configmanagement_v1.types.GitConfig):
            Git repo configuration for the cluster.
        source_format (str):
            Specifies whether the Config Sync Repo is
            in "hierarchical" or "unstructured" mode.
        enabled (bool):
            Enables the installation of ConfigSync.
            If set to true, ConfigSync resources will be
            created and the other ConfigSync fields will be
            applied if exist.
            If set to false, all other ConfigSync fields
            will be ignored, ConfigSync resources will be
            deleted.
            If omitted, ConfigSync resources will be managed
            depends on the presence of the git or oci field.

            This field is a member of `oneof`_ ``_enabled``.
        prevent_drift (bool):
            Set to true to enable the Config Sync admission webhook to
            prevent drifts. If set to ``false``, disables the Config
            Sync admission webhook and does not prevent drifts.
        oci (google.cloud.gkehub.configmanagement_v1.types.OciConfig):
            OCI repo configuration for the cluster
        metrics_gcp_service_account_email (str):
            The Email of the Google Cloud Service Account (GSA) used for
            exporting Config Sync metrics to Cloud Monitoring when
            Workload Identity is enabled. The GSA should have the
            Monitoring Metric Writer (roles/monitoring.metricWriter) IAM
            role. The Kubernetes ServiceAccount ``default`` in the
            namespace ``config-management-monitoring`` should be bound
            to the GSA.
    """

    git: "GitConfig" = proto.Field(
        proto.MESSAGE,
        number=7,
        message="GitConfig",
    )
    source_format: str = proto.Field(
        proto.STRING,
        number=8,
    )
    enabled: bool = proto.Field(
        proto.BOOL,
        number=10,
        optional=True,
    )
    prevent_drift: bool = proto.Field(
        proto.BOOL,
        number=11,
    )
    oci: "OciConfig" = proto.Field(
        proto.MESSAGE,
        number=12,
        message="OciConfig",
    )
    metrics_gcp_service_account_email: str = proto.Field(
        proto.STRING,
        number=15,
    )


class GitConfig(proto.Message):
    r"""Git repo configuration for a single cluster.

    Attributes:
        sync_repo (str):
            The URL of the Git repository to use as the
            source of truth.
        sync_branch (str):
            The branch of the repository to sync from.
            Default: master.
        policy_dir (str):
            The path within the Git repository that
            represents the top level of the repo to sync.
            Default: the root directory of the repository.
        sync_wait_secs (int):
            Period in seconds between consecutive syncs.
            Default: 15.
        sync_rev (str):
            Git revision (tag or hash) to check out.
            Default HEAD.
        secret_type (str):
            Type of secret configured for access to the
            Git repo. Must be one of ssh, cookiefile,
            gcenode, token, gcpserviceaccount or none. The
            validation of this is case-sensitive. Required.
        https_proxy (str):
            URL for the HTTPS proxy to be used when
            communicating with the Git repo.
        gcp_service_account_email (str):
            The Google Cloud Service Account Email used for auth when
            secret_type is gcpServiceAccount.
    """

    sync_repo: str = proto.Field(
        proto.STRING,
        number=1,
    )
    sync_branch: str = proto.Field(
        proto.STRING,
        number=2,
    )
    policy_dir: str = proto.Field(
        proto.STRING,
        number=3,
    )
    sync_wait_secs: int = proto.Field(
        proto.INT64,
        number=4,
    )
    sync_rev: str = proto.Field(
        proto.STRING,
        number=5,
    )
    secret_type: str = proto.Field(
        proto.STRING,
        number=6,
    )
    https_proxy: str = proto.Field(
        proto.STRING,
        number=7,
    )
    gcp_service_account_email: str = proto.Field(
        proto.STRING,
        number=8,
    )


class OciConfig(proto.Message):
    r"""OCI repo configuration for a single cluster

    Attributes:
        sync_repo (str):
            The OCI image repository URL for the package to sync from.
            e.g.
            ``LOCATION-docker.pkg.dev/PROJECT_ID/REPOSITORY_NAME/PACKAGE_NAME``.
        policy_dir (str):
            The absolute path of the directory that
            contains the local resources.  Default: the root
            directory of the image.
        sync_wait_secs (int):
            Period in seconds between consecutive syncs.
            Default: 15.
        secret_type (str):
            Type of secret configured for access to the
            Git repo.
        gcp_service_account_email (str):
            The Google Cloud Service Account Email used for auth when
            secret_type is gcpServiceAccount.
    """

    sync_repo: str = proto.Field(
        proto.STRING,
        number=1,
    )
    policy_dir: str = proto.Field(
        proto.STRING,
        number=2,
    )
    sync_wait_secs: int = proto.Field(
        proto.INT64,
        number=3,
    )
    secret_type: str = proto.Field(
        proto.STRING,
        number=4,
    )
    gcp_service_account_email: str = proto.Field(
        proto.STRING,
        number=5,
    )


class PolicyController(proto.Message):
    r"""Configuration for Policy Controller

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        enabled (bool):
            Enables the installation of Policy
            Controller. If false, the rest of
            PolicyController fields take no effect.
        template_library_installed (bool):
            Installs the default template library along
            with Policy Controller.

            This field is a member of `oneof`_ ``_template_library_installed``.
        audit_interval_seconds (int):
            Sets the interval for Policy Controller Audit
            Scans (in seconds). When set to 0, this disables
            audit functionality altogether.

            This field is a member of `oneof`_ ``_audit_interval_seconds``.
        exemptable_namespaces (MutableSequence[str]):
            The set of namespaces that are excluded from
            Policy Controller checks. Namespaces do not need
            to currently exist on the cluster.
        referential_rules_enabled (bool):
            Enables the ability to use Constraint
            Templates that reference to objects other than
            the object currently being evaluated.
        log_denies_enabled (bool):
            Logs all denies and dry run failures.
    """

    enabled: bool = proto.Field(
        proto.BOOL,
        number=1,
    )
    template_library_installed: bool = proto.Field(
        proto.BOOL,
        number=2,
        optional=True,
    )
    audit_interval_seconds: int = proto.Field(
        proto.INT64,
        number=3,
        optional=True,
    )
    exemptable_namespaces: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=4,
    )
    referential_rules_enabled: bool = proto.Field(
        proto.BOOL,
        number=5,
    )
    log_denies_enabled: bool = proto.Field(
        proto.BOOL,
        number=6,
    )


class HierarchyControllerConfig(proto.Message):
    r"""Configuration for Hierarchy Controller

    Attributes:
        enabled (bool):
            Whether Hierarchy Controller is enabled in
            this cluster.
        enable_pod_tree_labels (bool):
            Whether pod tree labels are enabled in this
            cluster.
        enable_hierarchical_resource_quota (bool):
            Whether hierarchical resource quota is
            enabled in this cluster.
    """

    enabled: bool = proto.Field(
        proto.BOOL,
        number=1,
    )
    enable_pod_tree_labels: bool = proto.Field(
        proto.BOOL,
        number=2,
    )
    enable_hierarchical_resource_quota: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class HierarchyControllerDeploymentState(proto.Message):
    r"""Deployment state for Hierarchy Controller

    Attributes:
        hnc (google.cloud.gkehub.configmanagement_v1.types.DeploymentState):
            The deployment state for open source HNC
            (e.g. v0.7.0-hc.0)
        extension (google.cloud.gkehub.configmanagement_v1.types.DeploymentState):
            The deployment state for Hierarchy Controller
            extension (e.g. v0.7.0-hc.1)
    """

    hnc: "DeploymentState" = proto.Field(
        proto.ENUM,
        number=1,
        enum="DeploymentState",
    )
    extension: "DeploymentState" = proto.Field(
        proto.ENUM,
        number=2,
        enum="DeploymentState",
    )


class HierarchyControllerVersion(proto.Message):
    r"""Version for Hierarchy Controller

    Attributes:
        hnc (str):
            Version for open source HNC
        extension (str):
            Version for Hierarchy Controller extension
    """

    hnc: str = proto.Field(
        proto.STRING,
        number=1,
    )
    extension: str = proto.Field(
        proto.STRING,
        number=2,
    )


class HierarchyControllerState(proto.Message):
    r"""State for Hierarchy Controller

    Attributes:
        version (google.cloud.gkehub.configmanagement_v1.types.HierarchyControllerVersion):
            The version for Hierarchy Controller
        state (google.cloud.gkehub.configmanagement_v1.types.HierarchyControllerDeploymentState):
            The deployment state for Hierarchy Controller
    """

    version: "HierarchyControllerVersion" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="HierarchyControllerVersion",
    )
    state: "HierarchyControllerDeploymentState" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="HierarchyControllerDeploymentState",
    )


class OperatorState(proto.Message):
    r"""State information for an ACM's Operator

    Attributes:
        version (str):
            The semenatic version number of the operator
        deployment_state (google.cloud.gkehub.configmanagement_v1.types.DeploymentState):
            The state of the Operator's deployment
        errors (MutableSequence[google.cloud.gkehub.configmanagement_v1.types.InstallError]):
            Install errors.
    """

    version: str = proto.Field(
        proto.STRING,
        number=1,
    )
    deployment_state: "DeploymentState" = proto.Field(
        proto.ENUM,
        number=2,
        enum="DeploymentState",
    )
    errors: MutableSequence["InstallError"] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="InstallError",
    )


class InstallError(proto.Message):
    r"""Errors pertaining to the installation of ACM

    Attributes:
        error_message (str):
            A string representing the user facing error
            message
    """

    error_message: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ConfigSyncState(proto.Message):
    r"""State information for ConfigSync

    Attributes:
        version (google.cloud.gkehub.configmanagement_v1.types.ConfigSyncVersion):
            The version of ConfigSync deployed
        deployment_state (google.cloud.gkehub.configmanagement_v1.types.ConfigSyncDeploymentState):
            Information about the deployment of
            ConfigSync, including the version of the various
            Pods deployed
        sync_state (google.cloud.gkehub.configmanagement_v1.types.SyncState):
            The state of ConfigSync's process to sync
            configs to a cluster
        errors (MutableSequence[google.cloud.gkehub.configmanagement_v1.types.ConfigSyncError]):
            Errors pertaining to the installation of
            Config Sync.
        rootsync_crd (google.cloud.gkehub.configmanagement_v1.types.ConfigSyncState.CRDState):
            The state of the RootSync CRD
        reposync_crd (google.cloud.gkehub.configmanagement_v1.types.ConfigSyncState.CRDState):
            The state of the Reposync CRD
        state (google.cloud.gkehub.configmanagement_v1.types.ConfigSyncState.State):
            The state of CS
            This field summarizes the other fields in this
            message.
    """

    class CRDState(proto.Enum):
        r"""CRDState representing the state of a CRD

        Values:
            CRD_STATE_UNSPECIFIED (0):
                CRD's state cannot be determined
            NOT_INSTALLED (1):
                CRD is not installed
            INSTALLED (2):
                CRD is installed
            TERMINATING (3):
                CRD is terminating (i.e., it has been deleted
                and is cleaning up)
            INSTALLING (4):
                CRD is installing
        """

        CRD_STATE_UNSPECIFIED = 0
        NOT_INSTALLED = 1
        INSTALLED = 2
        TERMINATING = 3
        INSTALLING = 4

    class State(proto.Enum):
        r"""

        Values:
            STATE_UNSPECIFIED (0):
                CS's state cannot be determined.
            CONFIG_SYNC_NOT_INSTALLED (1):
                CS is not installed.
            CONFIG_SYNC_INSTALLED (2):
                The expected CS version is installed
                successfully.
            CONFIG_SYNC_ERROR (3):
                CS encounters errors.
            CONFIG_SYNC_PENDING (4):
                CS is installing or terminating.
        """

        STATE_UNSPECIFIED = 0
        CONFIG_SYNC_NOT_INSTALLED = 1
        CONFIG_SYNC_INSTALLED = 2
        CONFIG_SYNC_ERROR = 3
        CONFIG_SYNC_PENDING = 4

    version: "ConfigSyncVersion" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="ConfigSyncVersion",
    )
    deployment_state: "ConfigSyncDeploymentState" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="ConfigSyncDeploymentState",
    )
    sync_state: "SyncState" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="SyncState",
    )
    errors: MutableSequence["ConfigSyncError"] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message="ConfigSyncError",
    )
    rootsync_crd: CRDState = proto.Field(
        proto.ENUM,
        number=5,
        enum=CRDState,
    )
    reposync_crd: CRDState = proto.Field(
        proto.ENUM,
        number=6,
        enum=CRDState,
    )
    state: State = proto.Field(
        proto.ENUM,
        number=7,
        enum=State,
    )


class ConfigSyncError(proto.Message):
    r"""Errors pertaining to the installation of Config Sync

    Attributes:
        error_message (str):
            A string representing the user facing error
            message
    """

    error_message: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ConfigSyncVersion(proto.Message):
    r"""Specific versioning information pertaining to ConfigSync's
    Pods

    Attributes:
        importer (str):
            Version of the deployed importer pod
        syncer (str):
            Version of the deployed syncer pod
        git_sync (str):
            Version of the deployed git-sync pod
        monitor (str):
            Version of the deployed monitor pod
        reconciler_manager (str):
            Version of the deployed reconciler-manager
            pod
        root_reconciler (str):
            Version of the deployed reconciler container
            in root-reconciler pod
        admission_webhook (str):
            Version of the deployed admission_webhook pod
    """

    importer: str = proto.Field(
        proto.STRING,
        number=1,
    )
    syncer: str = proto.Field(
        proto.STRING,
        number=2,
    )
    git_sync: str = proto.Field(
        proto.STRING,
        number=3,
    )
    monitor: str = proto.Field(
        proto.STRING,
        number=4,
    )
    reconciler_manager: str = proto.Field(
        proto.STRING,
        number=5,
    )
    root_reconciler: str = proto.Field(
        proto.STRING,
        number=6,
    )
    admission_webhook: str = proto.Field(
        proto.STRING,
        number=7,
    )


class ConfigSyncDeploymentState(proto.Message):
    r"""The state of ConfigSync's deployment on a cluster

    Attributes:
        importer (google.cloud.gkehub.configmanagement_v1.types.DeploymentState):
            Deployment state of the importer pod
        syncer (google.cloud.gkehub.configmanagement_v1.types.DeploymentState):
            Deployment state of the syncer pod
        git_sync (google.cloud.gkehub.configmanagement_v1.types.DeploymentState):
            Deployment state of the git-sync pod
        monitor (google.cloud.gkehub.configmanagement_v1.types.DeploymentState):
            Deployment state of the monitor pod
        reconciler_manager (google.cloud.gkehub.configmanagement_v1.types.DeploymentState):
            Deployment state of reconciler-manager pod
        root_reconciler (google.cloud.gkehub.configmanagement_v1.types.DeploymentState):
            Deployment state of root-reconciler
        admission_webhook (google.cloud.gkehub.configmanagement_v1.types.DeploymentState):
            Deployment state of admission-webhook
    """

    importer: "DeploymentState" = proto.Field(
        proto.ENUM,
        number=1,
        enum="DeploymentState",
    )
    syncer: "DeploymentState" = proto.Field(
        proto.ENUM,
        number=2,
        enum="DeploymentState",
    )
    git_sync: "DeploymentState" = proto.Field(
        proto.ENUM,
        number=3,
        enum="DeploymentState",
    )
    monitor: "DeploymentState" = proto.Field(
        proto.ENUM,
        number=4,
        enum="DeploymentState",
    )
    reconciler_manager: "DeploymentState" = proto.Field(
        proto.ENUM,
        number=5,
        enum="DeploymentState",
    )
    root_reconciler: "DeploymentState" = proto.Field(
        proto.ENUM,
        number=6,
        enum="DeploymentState",
    )
    admission_webhook: "DeploymentState" = proto.Field(
        proto.ENUM,
        number=7,
        enum="DeploymentState",
    )


class SyncState(proto.Message):
    r"""State indicating an ACM's progress syncing configurations to
    a cluster

    Attributes:
        source_token (str):
            Token indicating the state of the repo.
        import_token (str):
            Token indicating the state of the importer.
        sync_token (str):
            Token indicating the state of the syncer.
        last_sync (str):
            Deprecated: use last_sync_time instead. Timestamp of when
            ACM last successfully synced the repo The time format is
            specified in https://golang.org/pkg/time/#Time.String
        last_sync_time (google.protobuf.timestamp_pb2.Timestamp):
            Timestamp type of when ACM last successfully
            synced the repo
        code (google.cloud.gkehub.configmanagement_v1.types.SyncState.SyncCode):
            Sync status code
        errors (MutableSequence[google.cloud.gkehub.configmanagement_v1.types.SyncError]):
            A list of errors resulting from problematic
            configs. This list will be truncated after 100
            errors, although it is unlikely for that many
            errors to simultaneously exist.
    """

    class SyncCode(proto.Enum):
        r"""An enum representing Config Sync's status of syncing configs
        to a cluster.

        Values:
            SYNC_CODE_UNSPECIFIED (0):
                Config Sync cannot determine a sync code
            SYNCED (1):
                Config Sync successfully synced the git Repo
                with the cluster
            PENDING (2):
                Config Sync is in the progress of syncing a
                new change
            ERROR (3):
                Indicates an error configuring Config Sync,
                and user action is required
            NOT_CONFIGURED (4):
                Config Sync has been installed but not
                configured
            NOT_INSTALLED (5):
                Config Sync has not been installed
            UNAUTHORIZED (6):
                Error authorizing with the cluster
            UNREACHABLE (7):
                Cluster could not be reached
        """

        SYNC_CODE_UNSPECIFIED = 0
        SYNCED = 1
        PENDING = 2
        ERROR = 3
        NOT_CONFIGURED = 4
        NOT_INSTALLED = 5
        UNAUTHORIZED = 6
        UNREACHABLE = 7

    source_token: str = proto.Field(
        proto.STRING,
        number=1,
    )
    import_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    sync_token: str = proto.Field(
        proto.STRING,
        number=3,
    )
    last_sync: str = proto.Field(
        proto.STRING,
        number=4,
    )
    last_sync_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=7,
        message=timestamp_pb2.Timestamp,
    )
    code: SyncCode = proto.Field(
        proto.ENUM,
        number=5,
        enum=SyncCode,
    )
    errors: MutableSequence["SyncError"] = proto.RepeatedField(
        proto.MESSAGE,
        number=6,
        message="SyncError",
    )


class SyncError(proto.Message):
    r"""An ACM created error representing a problem syncing
    configurations

    Attributes:
        code (str):
            An ACM defined error code
        error_message (str):
            A description of the error
        error_resources (MutableSequence[google.cloud.gkehub.configmanagement_v1.types.ErrorResource]):
            A list of config(s) associated with the
            error, if any
    """

    code: str = proto.Field(
        proto.STRING,
        number=1,
    )
    error_message: str = proto.Field(
        proto.STRING,
        number=2,
    )
    error_resources: MutableSequence["ErrorResource"] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="ErrorResource",
    )


class ErrorResource(proto.Message):
    r"""Model for a config file in the git repo with an associated
    Sync error

    Attributes:
        source_path (str):
            Path in the git repo of the erroneous config
        resource_name (str):
            Metadata name of the resource that is causing
            an error
        resource_namespace (str):
            Namespace of the resource that is causing an
            error
        resource_gvk (google.cloud.gkehub.configmanagement_v1.types.GroupVersionKind):
            Group/version/kind of the resource that is
            causing an error
    """

    source_path: str = proto.Field(
        proto.STRING,
        number=1,
    )
    resource_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    resource_namespace: str = proto.Field(
        proto.STRING,
        number=3,
    )
    resource_gvk: "GroupVersionKind" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="GroupVersionKind",
    )


class GroupVersionKind(proto.Message):
    r"""A Kubernetes object's GVK

    Attributes:
        group (str):
            Kubernetes Group
        version (str):
            Kubernetes Version
        kind (str):
            Kubernetes Kind
    """

    group: str = proto.Field(
        proto.STRING,
        number=1,
    )
    version: str = proto.Field(
        proto.STRING,
        number=2,
    )
    kind: str = proto.Field(
        proto.STRING,
        number=3,
    )


class PolicyControllerState(proto.Message):
    r"""State for PolicyControllerState.

    Attributes:
        version (google.cloud.gkehub.configmanagement_v1.types.PolicyControllerVersion):
            The version of Gatekeeper Policy Controller
            deployed.
        deployment_state (google.cloud.gkehub.configmanagement_v1.types.GatekeeperDeploymentState):
            The state about the policy controller
            installation.
    """

    version: "PolicyControllerVersion" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="PolicyControllerVersion",
    )
    deployment_state: "GatekeeperDeploymentState" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="GatekeeperDeploymentState",
    )


class PolicyControllerVersion(proto.Message):
    r"""The build version of Gatekeeper Policy Controller is using.

    Attributes:
        version (str):
            The gatekeeper image tag that is composed of
            ACM version, git tag, build number.
    """

    version: str = proto.Field(
        proto.STRING,
        number=1,
    )


class GatekeeperDeploymentState(proto.Message):
    r"""State of Policy Controller installation.

    Attributes:
        gatekeeper_controller_manager_state (google.cloud.gkehub.configmanagement_v1.types.DeploymentState):
            Status of gatekeeper-controller-manager pod.
        gatekeeper_audit (google.cloud.gkehub.configmanagement_v1.types.DeploymentState):
            Status of gatekeeper-audit deployment.
    """

    gatekeeper_controller_manager_state: "DeploymentState" = proto.Field(
        proto.ENUM,
        number=1,
        enum="DeploymentState",
    )
    gatekeeper_audit: "DeploymentState" = proto.Field(
        proto.ENUM,
        number=2,
        enum="DeploymentState",
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from google.cloud.gkehub import gapic_version as package_version

__version__ = package_version.__version__


from google.cloud.gkehub_v1.services.gke_hub.async_client import GkeHubAsyncClient
from google.cloud.gkehub_v1.services.gke_hub.client import GkeHubClient
from google.cloud.gkehub_v1.types.feature import (
    CommonFeatureSpec,
    CommonFeatureState,
    Feature,
    FeatureResourceState,
    FeatureState,
    MembershipFeatureSpec,
    MembershipFeatureState,
    ScopeFeatureSpec,
    ScopeFeatureState,
)
from google.cloud.gkehub_v1.types.fleet import (
    BinaryAuthorizationConfig,
    CompliancePostureConfig,
    DefaultClusterConfig,
    Fleet,
    FleetLifecycleState,
    MembershipBinding,
    MembershipBindingLifecycleState,
    Namespace,
    NamespaceLifecycleState,
    RBACRoleBinding,
    RBACRoleBindingLifecycleState,
    Scope,
    ScopeLifecycleState,
    SecurityPostureConfig,
)
from google.cloud.gkehub_v1.types.membership import (
    Authority,
    GkeCluster,
    KubernetesMetadata,
    KubernetesResource,
    Membership,
    MembershipEndpoint,
    MembershipState,
    MonitoringConfig,
    ResourceManifest,
    ResourceOptions,
)
from google.cloud.gkehub_v1.types.service import (
    ConnectAgentResource,
    CreateFeatureRequest,
    CreateFleetRequest,
    CreateMembershipBindingRequest,
    CreateMembershipRBACRoleBindingRequest,
    CreateMembershipRequest,
    CreateScopeNamespaceRequest,
    CreateScopeRBACRoleBindingRequest,
    CreateScopeRequest,
    DeleteFeatureRequest,
    DeleteFleetRequest,
    DeleteMembershipBindingRequest,
    DeleteMembershipRBACRoleBindingRequest,
    DeleteMembershipRequest,
    DeleteScopeNamespaceRequest,
    DeleteScopeRBACRoleBindingRequest,
    DeleteScopeRequest,
    GenerateConnectManifestRequest,
    GenerateConnectManifestResponse,
    GenerateMembershipRBACRoleBindingYAMLRequest,
    GenerateMembershipRBACRoleBindingYAMLResponse,
    GetFeatureRequest,
    GetFleetRequest,
    GetMembershipBindingRequest,
    GetMembershipRBACRoleBindingRequest,
    GetMembershipRequest,
    GetScopeNamespaceRequest,
    GetScopeRBACRoleBindingRequest,
    GetScopeRequest,
    ListBoundMembershipsRequest,
    ListBoundMembershipsResponse,
    ListFeaturesRequest,
    ListFeaturesResponse,
    ListFleetsRequest,
    ListFleetsResponse,
    ListMembershipBindingsRequest,
    ListMembershipBindingsResponse,
    ListMembershipRBACRoleBindingsRequest,
    ListMembershipRBACRoleBindingsResponse,
    ListMembershipsRequest,
    ListMembershipsResponse,
    ListPermittedScopesRequest,
    ListPermittedScopesResponse,
    ListScopeNamespacesRequest,
    ListScopeNamespacesResponse,
    ListScopeRBACRoleBindingsRequest,
    ListScopeRBACRoleBindingsResponse,
    ListScopesRequest,
    ListScopesResponse,
    OperationMetadata,
    TypeMeta,
    UpdateFeatureRequest,
    UpdateFleetRequest,
    UpdateMembershipBindingRequest,
    UpdateMembershipRBACRoleBindingRequest,
    UpdateMembershipRequest,
    UpdateScopeNamespaceRequest,
    UpdateScopeRBACRoleBindingRequest,
    UpdateScopeRequest,
)

__all__ = (
    "GkeHubClient",
    "GkeHubAsyncClient",
    "CommonFeatureSpec",
    "CommonFeatureState",
    "Feature",
    "FeatureResourceState",
    "FeatureState",
    "MembershipFeatureSpec",
    "MembershipFeatureState",
    "ScopeFeatureSpec",
    "ScopeFeatureState",
    "BinaryAuthorizationConfig",
    "CompliancePostureConfig",
    "DefaultClusterConfig",
    "Fleet",
    "FleetLifecycleState",
    "MembershipBinding",
    "MembershipBindingLifecycleState",
    "Namespace",
    "NamespaceLifecycleState",
    "RBACRoleBinding",
    "RBACRoleBindingLifecycleState",
    "Scope",
    "ScopeLifecycleState",
    "SecurityPostureConfig",
    "Authority",
    "GkeCluster",
    "KubernetesMetadata",
    "KubernetesResource",
    "Membership",
    "MembershipEndpoint",
    "MembershipState",
    "MonitoringConfig",
    "ResourceManifest",
    "ResourceOptions",
    "ConnectAgentResource",
    "CreateFeatureRequest",
    "CreateFleetRequest",
    "CreateMembershipBindingRequest",
    "CreateMembershipRBACRoleBindingRequest",
    "CreateMembershipRequest",
    "CreateScopeNamespaceRequest",
    "CreateScopeRBACRoleBindingRequest",
    "CreateScopeRequest",
    "DeleteFeatureRequest",
    "DeleteFleetRequest",
    "DeleteMembershipBindingRequest",
    "DeleteMembershipRBACRoleBindingRequest",
    "DeleteMembershipRequest",
    "DeleteScopeNamespaceRequest",
    "DeleteScopeRBACRoleBindingRequest",
    "DeleteScopeRequest",
    "GenerateConnectManifestRequest",
    "GenerateConnectManifestResponse",
    "GenerateMembershipRBACRoleBindingYAMLRequest",
    "GenerateMembershipRBACRoleBindingYAMLResponse",
    "GetFeatureRequest",
    "GetFleetRequest",
    "GetMembershipBindingRequest",
    "GetMembershipRBACRoleBindingRequest",
    "GetMembershipRequest",
    "GetScopeNamespaceRequest",
    "GetScopeRBACRoleBindingRequest",
    "GetScopeRequest",
    "ListBoundMembershipsRequest",
    "ListBoundMembershipsResponse",
    "ListFeaturesRequest",
    "ListFeaturesResponse",
    "ListFleetsRequest",
    "ListFleetsResponse",
    "ListMembershipBindingsRequest",
    "ListMembershipBindingsResponse",
    "ListMembershipRBACRoleBindingsRequest",
    "ListMembershipRBACRoleBindingsResponse",
    "ListMembershipsRequest",
    "ListMembershipsResponse",
    "ListPermittedScopesRequest",
    "ListPermittedScopesResponse",
    "ListScopeNamespacesRequest",
    "ListScopeNamespacesResponse",
    "ListScopeRBACRoleBindingsRequest",
    "ListScopeRBACRoleBindingsResponse",
    "ListScopesRequest",
    "ListScopesResponse",
    "OperationMetadata",
    "TypeMeta",
    "UpdateFeatureRequest",
    "UpdateFleetRequest",
    "UpdateMembershipBindingRequest",
    "UpdateMembershipRBACRoleBindingRequest",
    "UpdateMembershipRequest",
    "UpdateScopeNamespaceRequest",
    "UpdateScopeRBACRoleBindingRequest",
    "UpdateScopeRequest",
)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import abc
from typing import Awaitable, Callable, Dict, Optional, Sequence, Union

import google.api_core
import google.auth  # type: ignore
import google.protobuf
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1, operations_v1
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.longrunning import operations_pb2  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.cloud.gkehub_v1 import gapic_version as package_version
from google.cloud.gkehub_v1.types import feature, fleet, membership, service

DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)

if hasattr(DEFAULT_CLIENT_INFO, "protobuf_runtime_version"):  # pragma: NO COVER
    DEFAULT_CLIENT_INFO.protobuf_runtime_version = google.protobuf.__version__


class GkeHubTransport(abc.ABC):
    """Abstract transport class for GkeHub."""

    AUTH_SCOPES = ("https://www.googleapis.com/auth/cloud-platform",)

    DEFAULT_HOST: str = "gkehub.googleapis.com"

    def __init__(
        self,
        *,
        host: str = DEFAULT_HOST,
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        api_audience: Optional[str] = None,
        **kwargs,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to (default: 'gkehub.googleapis.com').
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            credentials_file (Optional[str]): Deprecated. A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is mutually exclusive with credentials. This argument will be
                removed in the next major version of this library.
            scopes (Optional[Sequence[str]]): A list of scopes.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.
        """

        # Save the scopes.
        self._scopes = scopes
        if not hasattr(self, "_ignore_credentials"):
            self._ignore_credentials: bool = False

        # If no credentials are provided, then determine the appropriate
        # defaults.
        if credentials and credentials_file:
            raise core_exceptions.DuplicateCredentialArgs(
                "'credentials_file' and 'credentials' are mutually exclusive"
            )

        if credentials_file is not None:
            credentials, _ = google.auth.load_credentials_from_file(
                credentials_file,
                scopes=scopes,
                quota_project_id=quota_project_id,
                default_scopes=self.AUTH_SCOPES,
            )
        elif credentials is None and not self._ignore_credentials:
            credentials, _ = google.auth.default(
                scopes=scopes,
                quota_project_id=quota_project_id,
                default_scopes=self.AUTH_SCOPES,
            )
            # Don't apply audience if the credentials file passed from user.
            if hasattr(credentials, "with_gdch_audience"):
                credentials = credentials.with_gdch_audience(
                    api_audience if api_audience else host
                )

        # If the credentials are service account credentials, then always try to use self signed JWT.
        if (
            always_use_jwt_access
            and isinstance(credentials, service_account.Credentials)
            and hasattr(service_account.Credentials, "with_always_use_jwt_access")
        ):
            credentials = credentials.with_always_use_jwt_access(True)

        # Save the credentials.
        self._credentials = credentials

        # Save the hostname. Default to port 443 (HTTPS) if none is specified.
        if ":" not in host:
            host += ":443"
        self._host = host

    @property
    def host(self):
        return self._host

    def _prep_wrapped_messages(self, client_info):
        # Precompute the wrapped methods.
        self._wrapped_methods = {
            self.list_memberships: gapic_v1.method.wrap_method(
                self.list_memberships,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_bound_memberships: gapic_v1.method.wrap_method(
                self.list_bound_memberships,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_features: gapic_v1.method.wrap_method(
                self.list_features,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_membership: gapic_v1.method.wrap_method(
                self.get_membership,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_feature: gapic_v1.method.wrap_method(
                self.get_feature,
                default_timeout=None,
                client_info=client_info,
            ),
            self.create_membership: gapic_v1.method.wrap_method(
                self.create_membership,
                default_timeout=None,
                client_info=client_info,
            ),
            self.create_feature: gapic_v1.method.wrap_method(
                self.create_feature,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_membership: gapic_v1.method.wrap_method(
                self.delete_membership,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_feature: gapic_v1.method.wrap_method(
                self.delete_feature,
                default_timeout=None,
                client_info=client_info,
            ),
            self.update_membership: gapic_v1.method.wrap_method(
                self.update_membership,
                default_timeout=None,
                client_info=client_info,
            ),
            self.update_feature: gapic_v1.method.wrap_method(
                self.update_feature,
                default_timeout=None,
                client_info=client_info,
            ),
            self.generate_connect_manifest: gapic_v1.method.wrap_method(
                self.generate_connect_manifest,
                default_timeout=None,
                client_info=client_info,
            ),
            self.create_fleet: gapic_v1.method.wrap_method(
                self.create_fleet,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_fleet: gapic_v1.method.wrap_method(
                self.get_fleet,
                default_timeout=None,
                client_info=client_info,
            ),
            self.update_fleet: gapic_v1.method.wrap_method(
                self.update_fleet,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_fleet: gapic_v1.method.wrap_method(
                self.delete_fleet,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_fleets: gapic_v1.method.wrap_method(
                self.list_fleets,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_scope_namespace: gapic_v1.method.wrap_method(
                self.get_scope_namespace,
                default_timeout=None,
                client_info=client_info,
            ),
            self.create_scope_namespace: gapic_v1.method.wrap_method(
                self.create_scope_namespace,
                default_timeout=None,
                client_info=client_info,
            ),
            self.update_scope_namespace: gapic_v1.method.wrap_method(
                self.update_scope_namespace,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_scope_namespace: gapic_v1.method.wrap_method(
                self.delete_scope_namespace,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_scope_namespaces: gapic_v1.method.wrap_method(
                self.list_scope_namespaces,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_scope_rbac_role_binding: gapic_v1.method.wrap_method(
                self.get_scope_rbac_role_binding,
                default_timeout=None,
                client_info=client_info,
            ),
            self.create_scope_rbac_role_binding: gapic_v1.method.wrap_method(
                self.create_scope_rbac_role_binding,
                default_timeout=None,
                client_info=client_info,
            ),
            self.update_scope_rbac_role_binding: gapic_v1.method.wrap_method(
                self.update_scope_rbac_role_binding,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_scope_rbac_role_binding: gapic_v1.method.wrap_method(
                self.delete_scope_rbac_role_binding,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_scope_rbac_role_bindings: gapic_v1.method.wrap_method(
                self.list_scope_rbac_role_bindings,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_scope: gapic_v1.method.wrap_method(
                self.get_scope,
                default_timeout=None,
                client_info=client_info,
            ),
            self.create_scope: gapic_v1.method.wrap_method(
                self.create_scope,
                default_timeout=None,
                client_info=client_info,
            ),
            self.update_scope: gapic_v1.method.wrap_method(
                self.update_scope,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_scope: gapic_v1.method.wrap_method(
                self.delete_scope,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_scopes: gapic_v1.method.wrap_method(
                self.list_scopes,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_permitted_scopes: gapic_v1.method.wrap_method(
                self.list_permitted_scopes,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_membership_binding: gapic_v1.method.wrap_method(
                self.get_membership_binding,
                default_timeout=None,
                client_info=client_info,
            ),
            self.create_membership_binding: gapic_v1.method.wrap_method(
                self.create_membership_binding,
                default_timeout=None,
                client_info=client_info,
            ),
            self.update_membership_binding: gapic_v1.method.wrap_method(
                self.update_membership_binding,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_membership_binding: gapic_v1.method.wrap_method(
                self.delete_membership_binding,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_membership_bindings: gapic_v1.method.wrap_method(
                self.list_membership_bindings,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_membership_rbac_role_binding: gapic_v1.method.wrap_method(
                self.get_membership_rbac_role_binding,
                default_timeout=None,
                client_info=client_info,
            ),
            self.create_membership_rbac_role_binding: gapic_v1.method.wrap_method(
                self.create_membership_rbac_role_binding,
                default_timeout=None,
                client_info=client_info,
            ),
            self.update_membership_rbac_role_binding: gapic_v1.method.wrap_method(
                self.update_membership_rbac_role_binding,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_membership_rbac_role_binding: gapic_v1.method.wrap_method(
                self.delete_membership_rbac_role_binding,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_membership_rbac_role_bindings: gapic_v1.method.wrap_method(
                self.list_membership_rbac_role_bindings,
                default_timeout=None,
                client_info=client_info,
            ),
            self.generate_membership_rbac_role_binding_yaml: gapic_v1.method.wrap_method(
                self.generate_membership_rbac_role_binding_yaml,
                default_timeout=None,
                client_info=client_info,
            ),
        }

    def close(self):
        """Closes resources associated with the transport.

        .. warning::
             Only call this method if the transport is NOT shared
             with other clients - this may cause errors in other clients!
        """
        raise NotImplementedError()

    @property
    def operations_client(self):
        """Return the client designed to process long-running operations."""
        raise NotImplementedError()

    @property
    def list_memberships(
        self,
    ) -> Callable[
        [service.ListMembershipsRequest],
        Union[
            service.ListMembershipsResponse, Awaitable[service.ListMembershipsResponse]
        ],
    ]:
        raise NotImplementedError()

    @property
    def list_bound_memberships(
        self,
    ) -> Callable[
        [service.ListBoundMembershipsRequest],
        Union[
            service.ListBoundMembershipsResponse,
            Awaitable[service.ListBoundMembershipsResponse],
        ],
    ]:
        raise NotImplementedError()

    @property
    def list_features(
        self,
    ) -> Callable[
        [service.ListFeaturesRequest],
        Union[service.ListFeaturesResponse, Awaitable[service.ListFeaturesResponse]],
    ]:
        raise NotImplementedError()

    @property
    def get_membership(
        self,
    ) -> Callable[
        [service.GetMembershipRequest],
        Union[membership.Membership, Awaitable[membership.Membership]],
    ]:
        raise NotImplementedError()

    @property
    def get_feature(
        self,
    ) -> Callable[
        [service.GetFeatureRequest], Union[feature.Feature, Awaitable[feature.Feature]]
    ]:
        raise NotImplementedError()

    @property
    def create_membership(
        self,
    ) -> Callable[
        [service.CreateMembershipRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def create_feature(
        self,
    ) -> Callable[
        [service.CreateFeatureRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def delete_membership(
        self,
    ) -> Callable[
        [service.DeleteMembershipRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def delete_feature(
        self,
    ) -> Callable[
        [service.DeleteFeatureRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def update_membership(
        self,
    ) -> Callable[
        [service.UpdateMembershipRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def update_feature(
        self,
    ) -> Callable[
        [service.UpdateFeatureRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def generate_connect_manifest(
        self,
    ) -> Callable[
        [service.GenerateConnectManifestRequest],
        Union[
            service.GenerateConnectManifestResponse,
            Awaitable[service.GenerateConnectManifestResponse],
        ],
    ]:
        raise NotImplementedError()

    @property
    def create_fleet(
        self,
    ) -> Callable[
        [service.CreateFleetRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def get_fleet(
        self,
    ) -> Callable[
        [service.GetFleetRequest], Union[fleet.Fleet, Awaitable[fleet.Fleet]]
    ]:
        raise NotImplementedError()

    @property
    def update_fleet(
        self,
    ) -> Callable[
        [service.UpdateFleetRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def delete_fleet(
        self,
    ) -> Callable[
        [service.DeleteFleetRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def list_fleets(
        self,
    ) -> Callable[
        [service.ListFleetsRequest],
        Union[service.ListFleetsResponse, Awaitable[service.ListFleetsResponse]],
    ]:
        raise NotImplementedError()

    @property
    def get_scope_namespace(
        self,
    ) -> Callable[
        [service.GetScopeNamespaceRequest],
        Union[fleet.Namespace, Awaitable[fleet.Namespace]],
    ]:
        raise NotImplementedError()

    @property
    def create_scope_namespace(
        self,
    ) -> Callable[
        [service.CreateScopeNamespaceRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def update_scope_namespace(
        self,
    ) -> Callable[
        [service.UpdateScopeNamespaceRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def delete_scope_namespace(
        self,
    ) -> Callable[
        [service.DeleteScopeNamespaceRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def list_scope_namespaces(
        self,
    ) -> Callable[
        [service.ListScopeNamespacesRequest],
        Union[
            service.ListScopeNamespacesResponse,
            Awaitable[service.ListScopeNamespacesResponse],
        ],
    ]:
        raise NotImplementedError()

    @property
    def get_scope_rbac_role_binding(
        self,
    ) -> Callable[
        [service.GetScopeRBACRoleBindingRequest],
        Union[fleet.RBACRoleBinding, Awaitable[fleet.RBACRoleBinding]],
    ]:
        raise NotImplementedError()

    @property
    def create_scope_rbac_role_binding(
        self,
    ) -> Callable[
        [service.CreateScopeRBACRoleBindingRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def update_scope_rbac_role_binding(
        self,
    ) -> Callable[
        [service.UpdateScopeRBACRoleBindingRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def delete_scope_rbac_role_binding(
        self,
    ) -> Callable[
        [service.DeleteScopeRBACRoleBindingRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def list_scope_rbac_role_bindings(
        self,
    ) -> Callable[
        [service.ListScopeRBACRoleBindingsRequest],
        Union[
            service.ListScopeRBACRoleBindingsResponse,
            Awaitable[service.ListScopeRBACRoleBindingsResponse],
        ],
    ]:
        raise NotImplementedError()

    @property
    def get_scope(
        self,
    ) -> Callable[
        [service.GetScopeRequest], Union[fleet.Scope, Awaitable[fleet.Scope]]
    ]:
        raise NotImplementedError()

    @property
    def create_scope(
        self,
    ) -> Callable[
        [service.CreateScopeRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def update_scope(
        self,
    ) -> Callable[
        [service.UpdateScopeRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def delete_scope(
        self,
    ) -> Callable[
        [service.DeleteScopeRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def list_scopes(
        self,
    ) -> Callable[
        [service.ListScopesRequest],
        Union[service.ListScopesResponse, Awaitable[service.ListScopesResponse]],
    ]:
        raise NotImplementedError()

    @property
    def list_permitted_scopes(
        self,
    ) -> Callable[
        [service.ListPermittedScopesRequest],
        Union[
            service.ListPermittedScopesResponse,
            Awaitable[service.ListPermittedScopesResponse],
        ],
    ]:
        raise NotImplementedError()

    @property
    def get_membership_binding(
        self,
    ) -> Callable[
        [service.GetMembershipBindingRequest],
        Union[fleet.MembershipBinding, Awaitable[fleet.MembershipBinding]],
    ]:
        raise NotImplementedError()

    @property
    def create_membership_binding(
        self,
    ) -> Callable[
        [service.CreateMembershipBindingRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def update_membership_binding(
        self,
    ) -> Callable[
        [service.UpdateMembershipBindingRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def delete_membership_binding(
        self,
    ) -> Callable[
        [service.DeleteMembershipBindingRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def list_membership_bindings(
        self,
    ) -> Callable[
        [service.ListMembershipBindingsRequest],
        Union[
            service.ListMembershipBindingsResponse,
            Awaitable[service.ListMembershipBindingsResponse],
        ],
    ]:
        raise NotImplementedError()

    @property
    def get_membership_rbac_role_binding(
        self,
    ) -> Callable[
        [service.GetMembershipRBACRoleBindingRequest],
        Union[fleet.RBACRoleBinding, Awaitable[fleet.RBACRoleBinding]],
    ]:
        raise NotImplementedError()

    @property
    def create_membership_rbac_role_binding(
        self,
    ) -> Callable[
        [service.CreateMembershipRBACRoleBindingRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def update_membership_rbac_role_binding(
        self,
    ) -> Callable[
        [service.UpdateMembershipRBACRoleBindingRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def delete_membership_rbac_role_binding(
        self,
    ) -> Callable[
        [service.DeleteMembershipRBACRoleBindingRequest],
        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
    ]:
        raise NotImplementedError()

    @property
    def list_membership_rbac_role_bindings(
        self,
    ) -> Callable[
        [service.ListMembershipRBACRoleBindingsRequest],
        Union[
            service.ListMembershipRBACRoleBindingsResponse,
            Awaitable[service.ListMembershipRBACRoleBindingsResponse],
        ],
    ]:
        raise NotImplementedError()

    @property
    def generate_membership_rbac_role_binding_yaml(
        self,
    ) -> Callable[
        [service.GenerateMembershipRBACRoleBindingYAMLRequest],
        Union[
            service.GenerateMembershipRBACRoleBindingYAMLResponse,
            Awaitable[service.GenerateMembershipRBACRoleBindingYAMLResponse],
        ],
    ]:
        raise NotImplementedError()

    @property
    def kind(self) -> str:
        raise NotImplementedError()


__all__ = ("GkeHubTransport",)
